#pragma once

#include "SimHandler.h"
#include "ConnectionEntry.h"
#include "ConnectionSender.h"
#include "ConnectionReceiver.h"
#include "Address.h"
#include "PacketConnection.h"

class CConnectionLayer
{
public:
	enum EResult
	{
		SUCCESS = 0,							// Packet received and processed successfully
		FAIL_INCORRECT_PORT = 1,				// Packet arrived on the incorrect port
		FAIL_INVALID_DESTINATION_PACKET = 2,	// Packet with invalid destination is not a message packet
		FAIL_INVALID_DESTINATION_MESSAGE = 3,	// Message packet with invalid destination is not an OPEN message
		FAIL_CONNECTION_NOT_EXIST = 4,			// Packet destination is valid, but the connection does not exist,
		FAIL_NOT_ACCEPTED = 5,					// The upper layer did not accept the connection request
		FAIL_MAX_CONNECTIONS_REACHED = 6		// The number of maximum opened or half-opened connections has been reached
	};
private:
	// Simulator
	CSimHandler*		sim;

	// Connections
	CConnectionEntry*	connections;
	CConnectionEntry*	connectionsFree;
	CConnectionEntry*	connectionsAlloc;

	__uint32			connectionsMax;
	__uint32			connectionsNum;

	// Port
	__uint16			port;

	// Delegates
	IDelegate5<void, __uint16, __uint16, CAddress, __byte, CPacket*>*
						delegateSend;
	IDelegate2<void, CConnectionReceiver*, CPacket*>*
						delegateRecv;
	IDelegate2<bool, CAddress, CPacket*>*
						delegateAccept;
	IDelegate1<void, CConnectionSender*>*
						delegateAccepted;

	Delegate1<CConnectionLayer, void, CConnection*>*
						delegateDispose;

	// Sender
	__uint32			senderSegmentSize;

public:
	CConnectionLayer(
		CSimHandler*														sim,
		__uint16															port,
		__uint32															connectionsMax,
		IDelegate5<void, __uint16, __uint16, CAddress, __byte, CPacket*>*	delegateSend,
		IDelegate2<void, CConnectionReceiver*, CPacket*>*					delegateRecv,
		IDelegate2<bool, CAddress, CPacket*>*								delegateAccept,
		IDelegate1<void, CConnectionSender*>*								delegateAccepted,
		__uint32															senderSegmentSize
		);
	virtual ~CConnectionLayer();

	inline __uint32			ConnectionsMax() { return this->connectionsMax; }
	inline __uint32			ConnectionsNum() { return this->connectionsNum; }

	EResult					Recv(CAddress srcAddress, __uint16 srcPort, __uint16 dstPort, CPacketConnection* packet);
	EResult					Create(CAddress dstAddress, __uint16 dstPort, CConnectionReceiver** receiver);

	CConnection*			Get(__uint32 id, __uint32 idEntry);

private:
	__uint32				Alloc();
	void					Assign(CConnection* connection);
	void					Free(__uint32 entry);
	bool					IsAvailable();

	void					Dispose(CConnection* connection);
};