#include "Headers.h"
#include "Host.h"
#include "RouteHost.h"
#include "Console.h"

CHost::CHost(
	__uint32		id,
	CSimHandler*	sim,
	CAddress		address,
	CInfo*			info,
	CData*			data
	) : CNode(id, sim, address, 1, &CRouteHost::route, info)
{
	assert(data);

	// Data
	this->data = data;

	// Initialize layers
	this->layerIgmp = new CLayerIgmp(this->sim, this->address, this->info->NumGroups());
	assert_mem(this->layerIgmp);

	// Create layer connections
		// IP layer <-> IGMP layer
	(*this->layerIp->EventRecv()) += this->layerIgmp->DelegateRecv();
	(*this->layerIgmp->EventSend()) += this->layerIp->DelegateSend();
		// IGMP layer <-> IP multicast layer
	(*this->layerIgmp->EventJoin()) += this->layerIpMcast->DelegateJoin();
	(*this->layerIgmp->EventLeave()) += this->layerIpMcast->DelegateLeave();
	(*this->layerIgmp->EventLocalJoin()) += this->layerIpMcast->DelegateLocalJoin();
	(*this->layerIgmp->EventLocalLeave()) += this->layerIpMcast->DelegateLocalLeave();
}

CHost::~CHost()
{
	// Delete layers
	delete this->layerIgmp;
}

void CHost::Finalize()
{
	// Call the base class finalizer
	CNode::Finalize();

	// Call the finalizer for the layers
	this->layerIgmp->Finalize();
}