#pragma once

#include "Host.h"
#include "StreamClient.h"

class CHostClient : public CHost
{
public:
	enum EState
	{
		STATE_OFF = 0,
		STATE_ON_IDLE = 1,
		STATE_ON_WATCHING = 2
	};

protected:
	// Session
	EState			state;
	CChannel*		channel;

	__time			sessionTimeStart;
	__time			sessionTimeFinish;

	// Stream
	CStreamClient*	streamClient;

	// Stat
	__uint32		statFramesRecv;
	__uint32		statFramesDiscarded;
	__uint32		statFramesExpected;

public:
	CHostClient(
		__uint32		id,
		CSimHandler*	sim,
		CAddress		address,
		CInfo*			info,
		CData*			data,
		__bitrate		bw
		);
	virtual ~CHostClient();

	void					EventArrive();
	void					EventLeave();
	void					EventWatch(CChannel* channel, __time duration);

	virtual void			Finalize();

	inline __uint32			StatFramesRecv() { return this->statFramesRecv; }
	inline __uint32			StatFramesDiscarded() { return this->statFramesDiscarded; }
	inline __uint32			StatFramesExpected() { return this->statFramesExpected; }

protected:
	void					ChannelConnect(CChannel* channel);
	void					ChannelDisconnect();
	void					ChannelSwitch(CChannel* channel);

	virtual void			ChannelConnectUcast(CChannel* channel) = 0;
	virtual void			ChannelDisconnectUcast() = 0;

	virtual void			ChannelConnectMcast(CChannel* channel) = 0;
	virtual void			ChannelDisconnectMcast() = 0;
};