#include "Headers.h"
#include "HostClientPull.h"
#include "Debug.h"

CHostClientPull::CHostClientPull(
	__uint32		id,
	CSimHandler*	sim,
	CAddress		address,
	CInfoPull*		info,
	CData*			data,
	__bitrate		bw
	) : CHostClient(id, sim, address, info, data, bw)
{
	// Global
	this->info = info;

	// Delegates
	this->delegateRecv = new Delegate4<CHostClientPull, void, __uint32, CPacketIp*, CPacketUdp*, CPacket*>(this, &CHostClientPull::Recv);
	assert_mem(this->delegateRecv);

	this->delegateLink = new Delegate0<CHostClientPull, CLink*>(this, &CHostClientPull::Link);
	assert_mem(this->delegateLink);

	// Stream client push
	this->streamClient = new CStreamClientPull(
		this->sim,
		this->info,
		this->address,
		this->layerUdp->DelegateSend(),
		this->layerIgmp->DelegateJoin(),
		this->layerIgmp->DelegateLeave(),
		this->delegateLink,
		0);

	// Create layer connections
		// Layer UDP -> this
	(*this->layerUdp->EventRecv()) += this->delegateRecv;
}

CHostClientPull::~CHostClientPull()
{
	// Client
	delete this->streamClient;

	// Delegates
	delete this->delegateRecv;
	delete this->delegateLink;
}

void CHostClientPull::ChannelConnectMcast(CChannel* channel)
{
	// Start the client
	this->streamClient->Start(channel);

#ifdef MULTICAST
	// Multicast statistics
	this->info->MulticastJoin(this->id, channel->Id());
#endif
}

void CHostClientPull::ChannelDisconnectMcast()
{
	// Stop the client
	this->streamClient->Stop();

#ifdef MULTICAST
	// Multicast statistics
	this->info->MulticastLeave(this->id, channel->Id());
#endif
}

void CHostClientPull::ChannelConnectUcast(CChannel* channel)
{
	// Start the client
	this->streamClient->Start(channel);
}

void CHostClientPull::ChannelDisconnectUcast()
{
	// Stop the client
	this->streamClient->Stop();
}

void CHostClientPull::Recv(__uint32 entry, CPacketIp* ip, CPacketUdp* udp, CPacket* packet)
{
	// Process the received packets with the client
	this->streamClient->Recv(ip->Src(), ip->Dst(), udp->Src(), udp->Dst(), packet);
}


void CHostClientPull::SendStream(CAddress dst, CPacketStream* packet)
{
	// Send the stream packet to the UDP layer (TTL = 128)
//	(*this->layerUdp->DelegateSend())(this->info->PortStream(), this->info->PortStream(), dst, 128, packet);
}

void CHostClientPull::SendMessage(CAddress dst, CStreamMessage* message)
{
	// Send the message packet to the UDP layer (TTL = 128)
//	(*this->layerUdp->DelegateSend())(this->info->PortControl(), this->info->PortControl(), dst, 128, message);
}

void CHostClientPull::Finalize()
{
	// Call base class finalizer
	CHostClient::Finalize();

	// Call client finalizer
	this->streamClient->Finalize();
}