#include "Headers.h"
#include "HostClientPush.h"
#include "Debug.h"

CHostClientPush::CHostClientPush(
	__uint32		id,
	CSimHandler*	sim,
	CAddress		address,
	CInfoPush*		info,
	CData*			data,
	__bitrate		bw
	) : CHostClient(id, sim, address, info, data, bw)
{
	// Global
	this->info = info;

	// Delegates
	this->delegateRecv = new Delegate4<CHostClientPush, void, __uint32, CPacketIp*, CPacketUdp*, CPacket*>(this, &CHostClientPush::Recv);
	assert_mem(this->delegateRecv);

	this->delegateSendStream = new Delegate2<CHostClientPush, void, CAddress, CPacketStream*>(this, &CHostClientPush::SendStream);
	assert_mem(this->delegateSendStream);

	this->delegateSendMessage = new Delegate2<CHostClientPush, void, CAddress, CStreamMessage*>(this, &CHostClientPush::SendMessage);
	assert_mem(this->delegateSendMessage);

	this->delegateLink = new Delegate0<CHostClientPush, CLink*>(this, &CHostClientPush::Link);
	assert_mem(this->delegateLink);

	// Stream client push
	this->streamClient = new CStreamClientPush(
		this->sim,
		this->info,
		this->address,
		this->delegateSendStream,
		this->delegateSendMessage,
		this->layerIgmp->DelegateJoin(),
		this->layerIgmp->DelegateLeave(),
		this->delegateLink,
		0,
		bw);

	// Create layer connections
		// Layer UDP -> this
	(*this->layerUdp->EventRecv()) += this->delegateRecv;
}

CHostClientPush::~CHostClientPush()
{
	// Client
	delete this->streamClient;

	// Delegates
	delete this->delegateRecv;
	delete this->delegateSendStream;
	delete this->delegateSendMessage;
	delete this->delegateLink;
}

void CHostClientPush::ChannelConnectMcast(CChannel* channel)
{
	// Start the client
	this->streamClient->Start(channel);

#ifdef MULTICAST
	// Multicast statistics
	this->info->MulticastJoin(this->id, channel->Id());
#endif
}

void CHostClientPush::ChannelDisconnectMcast()
{
	// Stop the client
	this->streamClient->Stop();

#ifdef MULTICAST
	// Multicast statistics
	this->info->MulticastLeave(this->id, channel->Id());
#endif
}

void CHostClientPush::ChannelConnectUcast(CChannel* channel)
{
	// Start the client
	this->streamClient->Start(channel);
}

void CHostClientPush::ChannelDisconnectUcast()
{
	// Stop the client
	this->streamClient->Stop();
}

void CHostClientPush::Recv(__uint32 entry, CPacketIp* ip, CPacketUdp* udp, CPacket* packet)
{
	// Process the received packets with the client
	this->streamClient->Recv(ip->Src(), ip->Dst(), udp->Src(), udp->Dst(), packet);
}


void CHostClientPush::SendStream(CAddress dst, CPacketStream* packet)
{
	// Send the stream packet to the UDP layer (TTL = 128)
	(*this->layerUdp->DelegateSend())(this->info->PortStream(), this->info->PortStream(), dst, 128, packet);
}

void CHostClientPush::SendMessage(CAddress dst, CStreamMessage* message)
{
	// Send the message packet to the UDP layer (TTL = 128)
	(*this->layerUdp->DelegateSend())(this->info->PortControl(), this->info->PortControl(), dst, 128, message);
}

void CHostClientPush::Finalize()
{
	// Call base class finalizer
	CHostClient::Finalize();

	// Call client finalizer
	this->streamClient->Finalize();
}