#pragma once

#include "Node.h"
#include "ConnectionLayer.h"

class CHostFlow : public CNode
{
private:
	// Connections layer
	CConnectionLayer*		connectionLayer;

	// Delegates
	Delegate4<CHostFlow, void, __uint32, CPacketIp*, CPacketUdp*, CPacket*>*
							delegateRecv;

	Delegate2<CHostFlow, void, CConnectionReceiver*, CPacket*>*
							delegateConnectionRecv;
	Delegate2<CHostFlow, bool, CAddress, CPacket*>*
							delegateAccept;
	Delegate1<CHostFlow, void, CConnectionSender*>*
							delegateAccepted;

	Delegate2<CHostFlow, void, CConnection*, CConnection::EOpenResult>*
							delegateOpen;
	Delegate2<CHostFlow, void, CConnection*, CConnection::ECloseResult>*
							delegateClose;


	// Connections
	CConnectionSender*		senders[2];
	CConnectionReceiver*	receiver;
	__uint8					senderIndex;


	// Timer
	CTimer<CHostFlow>*		timers[2];

	// Info
	__bitrate				streamRate;
	double					packetRate;
	__time					packetInterval; 
	__uint32				packetSize;
	__uint32				connectionsMax;
	__uint16				port;
	__uint32				packetIndex[2];
	__uint32				packets;

	// Data
	FILE*					fileSender;
	FILE*					fileReceiver;

	// Delegates
	Delegate2<CHostFlow, void, CConnectionSender*, CPacket*>*
							delegateSenderSend;

public:
	CHostFlow(
		__uint32		id,
		CSimHandler*	sim,
		CAddress		address,
		CInfo*			info
		);
	virtual ~CHostFlow();

	void			Connect(CAddress dst);
	void			DisconnectSender(__uint8 sender);
	void			DisconnectReceiver();

	void			StartFlow(__uint8 sender);
	void			StopFlow(__uint8 sender);

	virtual void	Finalize();

private:
	void			Recv(__uint32 entry, CPacketIp* ip, CPacketUdp* udp, CPacket* packet);

	void			ConnectionRecv(CConnectionReceiver* connection, CPacket* packet);
	bool			ConnectionAccept(CAddress src, CPacket* packet);
	void			ConnectionAccepted(CConnectionSender* connection);

	void			ConnectionOpen(CConnection* connection, CConnection::EOpenResult result);
	void			ConnectionClose(CConnection* connection, CConnection::ECloseResult result);

	void			SenderSend(CConnectionSender* sender, CPacket* packet);

	void			Timer(CTimerInfo* info);
};
