#include "Headers.h"
#include "HostServerPull.h"

CHostServerPull::CHostServerPull(
	__uint32		id,
	CSimHandler*	sim,
	CAddress		address,
	CInfoPull*		info,
	CData*			data
	) : CHostServer(id, sim, address, info, data)
{
	// Global
	this->info = info;

	// Delegates
	this->delegateRecv = new Delegate4<CHostServerPull, void, __uint32, CPacketIp*, CPacketUdp*, CPacket*>(this, &CHostServerPull::Recv);
	assert_mem(this->delegateRecv);

	this->delegateLink = new Delegate0<CHostServerPull, CLink*>(this, &CHostServerPull::Link);
	assert_mem(this->delegateLink);

	this->delegateSendStream = new Delegate2<CHostServerPull, void, CAddress, CPacketStream*>(this, &CHostServerPull::SendStream);
	assert_mem(this->delegateSendStream);

	this->delegateSendMessage = new Delegate2<CHostServerPull, void, CAddress, CStreamMessage*>(this, &CHostServerPull::SendMessage);
	assert_mem(this->delegateSendMessage);

	// Create stream servers for multicast channels
	this->streamServersMcast = new CStreamServerPushMcast*[this->info->NumChannelsMulticast()];
	assert_mem(this->streamServersMcast);

	for(__uint32 index = 0; index < this->info->NumChannelsMulticast(); index++)
	{
		this->streamServersMcast[index] = new CStreamServerPushMcast(
			this->sim,
			this->info->StreamSource(this->info->IndexMulticastToGlobal(index)),
			this->delegateSendStream,
			this->layerIgmp->DelegateJoin(),
			this->layerIgmp->DelegateLeave(),
			0
			);
		assert_mem(this->streamServersMcast[index]);
	}

	// Create the stream server for unicast channels
	this->streamServerUcast = new CStreamServerPull(
		this->sim,
		this->info,
		this->layerUdp->DelegateSend(),
		this->delegateLink
		);
	assert_mem(this->streamServerUcast);

	// Create layer connections
		// Layer UDP -> this
	(*this->layerUdp->EventRecv()) += this->delegateRecv;
}

CHostServerPull::~CHostServerPull()
{
	// Delete stream servers
	for(__uint32 index = 0; index < this->info->NumChannelsMulticast(); index++)
		delete this->streamServersMcast[index];
	delete[] this->streamServersMcast;

	delete this->streamServerUcast;

	// Delegates
	delete this->delegateRecv;
	delete this->delegateLink;
	delete this->delegateSendStream;
	delete this->delegateSendMessage;
}

void CHostServerPull::Start()
{
	// Start the multicast channel servers
	for(__uint32 index = 0; index < this->info->NumChannelsMulticast(); index++)
		this->streamServersMcast[index]->Start();

	// Start the unicast channels server
	this->streamServerUcast->Start();
}

void CHostServerPull::Stop()
{
	// Start the multicast channel servers
	for(__uint32 index = 0; index < this->info->NumChannelsMulticast(); index++)
		this->streamServersMcast[index]->Stop();

	// Start the unicast channels server
	this->streamServerUcast->Stop();
}

void CHostServerPull::Finalize()
{
	// Call base class finalizer
	CHostServer::Finalize();

	// Call stream servers finalizer
	for(__uint32 index = 0; index < this->info->NumChannelsMulticast(); index++)
		this->streamServersMcast[index]->Finalize();
	this->streamServerUcast->Finalize();
}

void CHostServerPull::Recv(__uint32 entry, CPacketIp* ip, CPacketUdp* udp, CPacket* packet)
{
	// Pass all received packets to the unicast server
	this->streamServerUcast->Recv(ip->Src(), udp->Src(), udp->Dst(), packet);
}

void CHostServerPull::SendStream(CAddress dst, CPacketStream* packet)
{
	// Send stream packet to the lower layer (TTL = 128)
	(*this->layerUdp->DelegateSend())(this->info->PortStream(), this->info->PortStream(), dst, 128, packet);
}

void CHostServerPull::SendMessage(CAddress dst, CStreamMessage* message)
{
	// Send message packet to the lower layer (TTL = 128)
//	(*this->layerUdp->DelegateSend())(this->info->PortControl(), this->info->PortControl(), dst, 128, message);
}
