#pragma once

#include "Layer.h"
#include "LayerIpMcastGroup.h"
#include "LayerIpMcastMembership.h"
#include "PacketIp.h"
#include "PacketIgmp.h"

class CLayerIgmp : public CLayer
{
public:
	enum EIgmpType
	{
		IGMP_HOST = 0,
		IGMP_ROUTER = 1
	};
	
private:
	CAddress												address;
	__uint32												numGroups;
	EIgmpType												type;
	
	CLayerIpMcastMembership*								memberships;	// Group memberships initiated locally
	CLayerIpMcastGroup*										groups;			// Group memberships initiated remotely

	Delegate2<CLayerIgmp, void, __uint32, CPacketIp*>*		delegateRecv;

	Delegate2<CLayerIgmp, void, CAddress, __uint32>*		delegateJoin;
	Delegate1<CLayerIgmp, void, CAddress>*					delegateLeave;

	Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*	eventJoin;
	Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*	eventLeave;

	Event2<void, __uint32, CAddress>*						eventLocalJoin;
	Event2<void, __uint32, CAddress>*						eventLocalLeave;

	Event2<void, __uint32, CPacketIp*>*						eventSend;

	double													statEntries;
	__uint32												statEntriesNum;
	__time													statEntriesLast;

public:
	CLayerIgmp(
		CSimHandler*	sim,
		CAddress		address,
		__uint32		numGroups,
		EIgmpType		type = IGMP_HOST
		);
	virtual ~CLayerIgmp();

	inline IDelegate2<void, __uint32, CPacketIp*>*
		DelegateRecv() { return this->delegateRecv; }

	inline IDelegate2<void, CAddress, __uint32>*
		DelegateJoin() { return this->delegateJoin; }
	inline IDelegate1<void, CAddress>*
		DelegateLeave() { return this->delegateLeave; }

	inline Event2<void, __uint32, CPacketIp*>*
		EventSend() { return this->eventSend; }

	inline Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*
		EventJoin() { return this->eventJoin; }
	inline Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*
		EventLeave() { return this->eventLeave; }

	inline Event2<void, __uint32, CAddress>*
		EventLocalJoin() { return this->eventLocalJoin; }
	inline Event2<void, __uint32, CAddress>*
		EventLocalLeave() { return this->eventLocalLeave; }

	inline double	StatEntries() { return this->statEntries; }

	virtual void	Finalize();

	void			Join(CAddress address, __uint32 entry);
	void			Leave(CAddress address);

private:
	void			Recv(__uint32 entry, CPacketIp* packet);
	void			RecvIgmpAllSystems(__uint32 entry, CAddress src, CAddress dst, CPacketIgmp* packet);
	void			RecvIgmpAllRouters(__uint32 entry, CAddress src, CAddress dst, CPacketIgmp* packet);
	void			RecvIgmpOther(__uint32 entry, CAddress src, CAddress dst, CPacketIgmp* packet);
	void			RecvIgmpJoinGroup(__uint32 entry, CAddress src, CAddress dst, CPacketIgmp* packet);
	void			RecvIgmpLeaveGroup(__uint32 entry, CAddress src, CAddress dst, CPacketIgmp* packet);
	
	void			Send(__uint32 entry, CAddress dst, __byte ttl, CPacket* payload);
};