#include "Headers.h"
#include "LayerIp.h"

CLayerIp::CLayerIp(CSimHandler* sim) : CLayer(sim)
{
	this->delegateRecv = new Delegate2<CLayerIp, void, __uint32, CPacket*>(this, &CLayerIp::Recv);
	assert_mem(this->delegateRecv);

	this->eventRecv = new Event2<void, __uint32, CPacketIp*>();
	assert_mem(this->eventRecv);

	this->delegateSend = new Delegate2<CLayerIp, void, __uint32, CPacketIp*>(this, &CLayerIp::Send);
	assert_mem(this->delegateSend);

	this->eventSend = new Event2<void, __uint32, CPacket*>();
	assert_mem(this->eventSend);
}

CLayerIp::~CLayerIp()
{
	delete this->delegateRecv;
	delete this->eventRecv;

	delete this->delegateSend;
	delete this->eventSend;
}

void CLayerIp::Recv(__uint32 entry, CPacket* packet)
{
	// Verify the packet is IP
	if(NULL == packet) return;
	if(packet->Type() != PACKET_TYPE_IP) return;

	// Get the IP packet
	CPacketIp* ip = type_cast<CPacketIp*>(packet);

	// Decrement the TTL
	ip->DecTtl();

#ifdef LOG_LAYER
	printf("\n\tT = %7.3lf IP RECV entry=%u bytes=%u", this->sim->Time(), entry, packet->Size());
#endif

	// Send the packet to the upper layers
	(*this->eventRecv)(entry, ip);
}

void CLayerIp::Send(__uint32 entry, CPacketIp* packet)
{
	if(NULL == packet) return;

#ifdef LOG_LAYER
	printf("\n\tT = %7.3lf IP SEND entry=%u bytes=%u", this->sim->Time(), entry, packet->Size());
#endif

	// Send the packer to the lower layer
	(*this->eventSend)(entry, packet);
}