#include "Headers.h"
#include "LayerIpLocal.h"

CLayerIpLocal::CLayerIpLocal(
	CSimHandler*	sim,
	CAddress		address,
	CRoute*			route
	) : CLayer(sim)
{
	this->address = address;
	this->route = route;

	// Delegates
	this->delegateRecv = new Delegate2<CLayerIpLocal, void, __uint32, CPacketIp*>(this, &CLayerIpLocal::Recv);
	assert_mem(this->delegateRecv);

	this->delegateRecvMcast = new Delegate2<CLayerIpLocal, void, __uint32, CPacketIp*>(this, &CLayerIpLocal::RecvMcast);
	assert_mem(this->delegateRecvMcast);

	this->delegateSend = new Delegate3<CLayerIpLocal, void, CAddress, __byte, CPacket*>(this, &CLayerIpLocal::Send);
	assert_mem(this->delegateSend);

	// Events
	this->eventRecv = new Event3<void, __uint32, CPacketIp*, CPacket*>();
	assert_mem(this->eventRecv);

	this->eventSend = new Event2<void, __uint32, CPacketIp*>();
	assert_mem(this->eventSend);

	this->eventSendMcast = new Event1<void, CPacketIp*>();
	assert_mem(this->eventSendMcast);
}

CLayerIpLocal::~CLayerIpLocal()
{
	// Delegates
	delete this->delegateRecv;
	delete this->delegateRecvMcast;
	delete this->delegateSend;

	// Events
	delete this->eventRecv;
	delete this->eventSend;
	delete this->eventSendMcast;
}

void CLayerIpLocal::Recv(__uint32 entry, CPacketIp* packet)
{
	// Verify the packet destination is the current address
	if(NULL == packet) return;
	if(packet->Dst() != this->address) return;

#ifdef LOG_LAYER
	printf("\n\tT = %7.3lf IP_LOCAL RECV local=%u entry=%u bytes=%u src=%u dst=%u", this->sim->Time(), this->address, entry, packet->Size(), packet->Src(), packet->Dst());
#endif

	// Send the packet payload to the upper layers
	(*this->eventRecv)(entry, packet, packet->Payload());
}

void CLayerIpLocal::RecvMcast(__uint32 entry, CPacketIp* packet)
{
	// Verify the packet destination is a multicast address
	if(NULL == packet) return;
	if(!packet->Dst().IsMulticast()) return;

#ifdef LOG_LAYER
	printf("\n\tT = %7.3lf IP_LOCAL RECV_MCAST local=%u entry=%u bytes=%u src=%u dst=%u", this->sim->Time(), this->address, entry, packet->Size(), packet->Src(), packet->Dst());
#endif

	// Send the packet payload to the upper layers
	(*this->eventRecv)(entry, packet, packet->Payload());
}

void CLayerIpLocal::Send(CAddress dst, __byte ttl, CPacket *payload)
{
	// Create a new IP packet
	CPacketIp* packet = new CPacketIp(this->address, dst, ttl, payload);
	assert_mem(packet);

#ifdef LOG_LAYER
	printf("\n\tT = %7.3lf IP_LOCAL SEND local=%u bytes=%u src=%u dst=%u", this->sim->Time(), this->address, packet->Size(), packet->Src(), packet->Dst());
#endif

	// If destination is multicast
	if(dst.IsMulticast())
	{
		// Send the packet to the lower multicast layer
		(*this->eventSendMcast)(packet);
	}
	// If destination is unicast
	else
	{
		// Calculate the link to send the packet
		__uint32 link = this->route->Forward(this->address, dst);

		// Send the packet to the lower layer
		(*this->eventSend)(link, packet);
	}
}