#pragma once

#include "Layer.h"
#include "LayerIpMcastGroup.h"
#include "LayerIpMcastMembership.h"
#include "PacketIp.h"

class CLayerIpMcast : public CLayer
{
private:
	__uint32																	numGroups;
	CLayerIpMcastGroup*															groups;
	CLayerIpMcastMembership*													memberships;

	// Delegates
	Delegate2<CLayerIpMcast, void, __uint32, CPacketIp*>*						delegateRecv;
	Delegate1<CLayerIpMcast, void, CPacketIp*>*									delegateSend;

	Delegate3<CLayerIpMcast, void, __uint32, CAddress, CLayerIpMcastGroup*>*	delegateJoin;
	Delegate3<CLayerIpMcast, void, __uint32, CAddress, CLayerIpMcastGroup*>*	delegateLeave;

	Delegate2<CLayerIpMcast, void, __uint32, CAddress>*							delegateLocalJoin;
	Delegate2<CLayerIpMcast, void, __uint32, CAddress>*							delegateLocalLeave;

	// Events
	Event2<void, __uint32, CPacketIp*>*											eventRecv;
	Event2<void, __uint32, CPacketIp*>*											eventSend;

public:
	CLayerIpMcast(
		CSimHandler*	sim,
		__uint32		numGroups
		);
	virtual ~CLayerIpMcast();

	inline IDelegate2<void, __uint32, CPacketIp*>*
		DelegateRecv() { return this->delegateRecv; }
	inline IDelegate1<void, CPacketIp*>*
		DelegateSend() { return this->delegateSend; }

	inline IDelegate3<void, __uint32, CAddress, CLayerIpMcastGroup*>*
		DelegateJoin() { return this->delegateJoin; }
	inline IDelegate3<void, __uint32, CAddress, CLayerIpMcastGroup*>*
		DelegateLeave() { return this->delegateLeave; }

	inline Delegate2<CLayerIpMcast, void, __uint32, CAddress>*
		DelegateLocalJoin() { return this->delegateLocalJoin; }
	inline Delegate2<CLayerIpMcast, void, __uint32, CAddress>*
		DelegateLocalLeave() { return this->delegateLocalLeave; }

	inline Event2<void, __uint32, CPacketIp*>*
		EventRecv() { return this->eventRecv; }
	inline Event2<void, __uint32, CPacketIp*>*
		EventSend() { return this->eventSend; }

	virtual void	Finalize() { }

private:
	void			Recv(__uint32 entry, CPacketIp* packet);
	void			Send(CPacketIp* packet);
	
	void			Join(__uint32 entry, CAddress address, CLayerIpMcastGroup* groupSender);
	void			Leave(__uint32 entry, CAddress address, CLayerIpMcastGroup* groupSender);

	void			LocalJoin(__uint32 entry, CAddress address);
	void			LocalLeave(__uint32 entry, CAddress address);
};