#include "Headers.h"
#include "LayerIpRoute.h"

CLayerIpRoute::CLayerIpRoute(
	CSimHandler*	sim,
	CAddress		address,
	CRoute*			route
	) : CLayer(sim)
{
	this->address = address;
	this->route = route;

	// Delegate and event
	this->delegateRecv = new Delegate2<CLayerIpRoute, void, __uint32, CPacketIp*>(this, &CLayerIpRoute::Recv);
	assert_mem(this->delegateRecv);
	
	this->eventSend = new Event2<void, __uint32, CPacketIp*>();
	assert_mem(this->eventSend);
}

CLayerIpRoute::~CLayerIpRoute()
{
	delete this->delegateRecv;
	delete this->eventSend;
}

void CLayerIpRoute::Recv(__uint32 entry, CPacketIp* packet)
{
	// Forward all unicast packets that are not address to local address and have a TTL greater than zero
	if(packet->Ttl() == 0) return;
	if(packet->Dst().IsMulticast()) return;
	if(packet->Dst() == this->address) return;

#ifdef LOG_LAYER
	printf("\n\tT = %7.3lf IP_ROUTE RECV local=%u bytes=%u src=%u dst=%u", this->sim->Time(), this->address, packet->Size(), packet->Src(), packet->Dst());
#endif

	__uint32 link = this->route->Forward(this->address, packet->Dst());

	(*this->eventSend)(link, type_cast<CPacketIp*>(packet->Copy()));
}