#pragma once

#include "Layer.h"
#include "LayerPimSmMembership.h"
#include "LayerIpMcastGroup.h"
#include "Route.h"
#include "PacketIp.h"
#include "PacketPimSm.h"
#include "PacketPimSmJoinPrune.h"

class CLayerPimSm : public CLayer
{
private:
	__uint32																numGroups;
	CAddress																address;
	CAddress*																rp;
	CRoute*																	route;

	CLayerPimSmMembership*													memberships;
	CLayerIpMcastGroup*														groups;

	Delegate2<CLayerPimSm, void, __uint32, CPacketIp*>*						delegateRecv;

	Delegate3<CLayerPimSm, void, __uint32, CAddress, CLayerIpMcastGroup*>*	delegateJoin;
	Delegate3<CLayerPimSm, void, __uint32, CAddress, CLayerIpMcastGroup*>*	delegateLeave;

	Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*					eventJoin;
	Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*					eventLeave;

	Event2<void, __uint32, CPacketIp*>*										eventSend;

	double																	statEntries;
	__uint32																statEntriesNum;
	__time																	statEntriesLast;

public:
	CLayerPimSm(
		CSimHandler*	sim,
		__uint32		numGroups,
		CAddress		address,
		CAddress*		rp,
		CRoute*			route
		);
	virtual ~CLayerPimSm();

	inline Delegate2<CLayerPimSm, void, __uint32, CPacketIp*>*
		DelegateRecv() { return this->delegateRecv;}

	inline Delegate3<CLayerPimSm, void, __uint32, CAddress, CLayerIpMcastGroup*>*
		DelegateJoin() { return this->delegateJoin; }
	inline Delegate3<CLayerPimSm, void, __uint32, CAddress, CLayerIpMcastGroup*>*
		DelegateLeave() { return this->delegateLeave; }

	inline Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*
		EventJoin() { return this->eventJoin; }
	inline Event3<void, __uint32, CAddress, CLayerIpMcastGroup*>*
		EventLeave() { return this->eventLeave; }

	inline Event2<void, __uint32, CPacketIp*>*
		EventSend() { return this->eventSend; }

	inline double	StatEntries() { return this->statEntries; }

	virtual void	Finalize();

private:
	void			Recv(__uint32 entry, CPacketIp* packet);
	void			RecvPimSm(__uint32 entry, CAddress src, CAddress dst, CPacketPimSm* packet);
	void			RecvPimSmJoinPrune(__uint32 entry, CAddress src, CAddress dst, CPacketPimSmJoinPrune* packet);

	void			Send(__uint32 entry, CAddress dst, __byte ttl, CPacket* payload);

	void			Join(__uint32 entry, CAddress address, CLayerIpMcastGroup* igmp);
	void			Leave(__uint32 entry, CAddress address, CLayerIpMcastGroup* igmp);

	void			JoinGroup(__uint32 group, CAddress address);
	void			LeaveGroup(__uint32 group, CAddress address);
};