#pragma once

#include "Object.h"
#include "Codes.h"
#include "LinkQueue.h"

class CLink : public CObject
{
protected:
	__uint32		id;
	CObject*		nodes[2];
	__uint32		nodeEntries[2];
	CLinkQueue*		queues[2][2];
	__uint32		currentNode;

	__bitrate		bandwidth[2];
	__time			delay[2];
	
	double			statUtil[2][2];
	__uint64		statPackets[2][2];
	__uint64		statDiscard[2][2];
	__uint64		statBits[2][2];
	double			statQueue[2][2];
	__time			statQueueLast[2][2];

	__time			meterTime[2];
	__bitrate		meterUtil[2];

	static double	meterSmoothFactor;

public:
	CLink(
		__uint32		id,
		CSimHandler*	sim,
		__uint32		queue,
		__bitrate		bandwidth0,
		__bitrate		bandwidth1,
		__time			delay0,
		__time			delay1
		);
	virtual ~CLink();

	inline __uint32			Id() { return this->id; }
	inline __bitrate		Bandwidth(__byte entry) { return this->bandwidth[entry]; }

	__uint32				AddNode(CObject* node, __uint32 entry);
	inline CObject*			Node(__uint32 index) { assert(index < 2); return this->nodes[index]; }
	inline __uint32			NodeEntry(__uint32 index) { assert(index < 2); return this->nodeEntries[index]; }

	virtual ENetworkCode	Recv(CObject* sender, __uint32 entry, CPacket* packet);
	void					Delay(__uint32 entry, CPacket* packet);
	void					Send(__uint32 entry, CPacket* packet);

	virtual void			Finalize();

	inline CLinkQueue*		Queue(__uint8 tos, __uint8 entry) { return this->queues[tos][entry]; }

	__bitrate				MeterUtil(__uint8 entry);
	__uint32				MeterQueue(__uint8 tos, __uint8 entry) { return this->queues[tos][entry]->Packets(); }

	inline double			StatUtilization(__byte tos, __byte entry) { return this->statUtil[tos][entry]; }
	inline __uint64			StatPackets(__byte tos, __byte entry) { return this->statPackets[tos][entry]; }
	inline __uint64			StatBits(__byte tos, __byte entry) { return this->statBits[tos][entry]; }
	inline __uint64			StatDiscard(__byte tos, __byte entry) { return this->statDiscard[tos][entry]; }
	inline double			StatQueue(__byte tos, __byte entry) { return this->statQueue[tos][entry]; }
};