#include "Headers.h"
#include "LinkQueue.h"

CLinkQueue::CLinkQueue(
	__uint32	size
	)
{
	assert(size);
	
	this->size = size;
	this->queue = new CPacket*[this->size];
	this->time = new __time[this->size];
	this->ptr = 0;
	this->count = 0;
	this->usage = 0;

	assert_mem(this->queue);
}

CLinkQueue::~CLinkQueue()
{
	// Delete packets stored in the queue (start from 1, because event 0 has an associated queue event and will be deleted with the event)
	for(__uint32 index = 1; index < this->count; index++)
	{
		__uint32 idx = (this->ptr + index) % this->size;

		this->queue[idx]->Delete();
		delete this->queue[idx];
	}

	delete[] this->queue;
	delete[] this->time;
}

void CLinkQueue::Add(__time time, CPacket* packet)
{
	assert(packet);
	assert(this->count < this->size);

	__uint32 index = (this->ptr + this->count) % this->size;

	this->queue[index] = packet;
	this->time[index] = time;
	this->count++;
	this->usage += packet->Size();
}

void CLinkQueue::Remove()
{
	assert(this->count > 0);

	this->usage -= this->queue[this->ptr]->Size();
	this->ptr = (this->ptr + 1) % this->size;
	this->count--;
}