#include "Headers.h"
#include "SimulatorPush.h"
#include "SimulatorPushMulti.h"
#include "SimulatorPull.h"
#include "SimulatorNetUcast.h"
#include "SimulatorNetMcast.h"
#include "SimulatorFlow.h"
#include "Rand.h"
#include "Console.h"

#pragma warning(disable : 4996)

int main(int argc, char* argv[])
{
#ifdef _WINDOWS
    // Send all reports to STDOUT
	_CrtSetReportMode( _CRT_WARN, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_WARN, _CRTDBG_FILE_STDOUT );
	//_CrtSetReportMode( _CRT_ERROR, _CRTDBG_MODE_FILE );
	//_CrtSetReportFile( _CRT_ERROR, _CRTDBG_FILE_STDOUT );
	_CrtSetReportMode( _CRT_ASSERT, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_ASSERT, _CRTDBG_FILE_STDOUT );
#endif

	__uint32 seed = CRand::Init();

	char option = 0;
	char* simName = "";
	int argCount = argc;
	char** argValue = argv;

	// Check if the simulator is selected in command line
	if(argc >= 3)
	{
		// If the first argument is '-s', get the simulator from the second parameter
		if(strcmp(argv[1], "-s") == 0)
		{
			option = argv[2][0];
			simName = argv[3];

			// Decrease arguments by two
			argCount = argc - 3;
			argValue = argv + 3;
		}
	}

	if(0 == option)
	{
		printf("\n\nSelect simulator or use \'-s\' switch in the command line:\
			   \n\t 1 - Push\
			   \n\t 2 - Push multi\
			   \n\t 3 - Pull\
			   \n\t 4 - Network unicast\
			   \n\t 5 - Network multicast\
			   \n\t 6 - Flow connection / congestion control\
			   \n\nPress a key to continue...");
		option = _getch();
	}
	else
	{
		printf("\nRandom seed: %u", seed);
	}

	switch(option)
	{
	case '1':
		{
			CSimulatorPush* simulator = new CSimulatorPush();
			simulator->Run(simName, argCount, argValue);
			delete simulator;
		}
		break;
	case '2':
		{
			CSimulatorPushMulti* simulator = new CSimulatorPushMulti();
			simulator->Run(simName, argCount, argValue);
			delete simulator;
		}
		break;
	case '3':
		{
			CSimulatorPull* simulator = new CSimulatorPull();
			simulator->Run(simName, argCount, argValue);
			delete simulator;
		}
		break;
	case '4':
		{
			CSimulatorNetUcast* simulator = new CSimulatorNetUcast();
			simulator->Run(simName, argCount, argValue);
			delete simulator;
		}
		break;
	case '5':
		{
			CSimulatorNetMcast* simulator = new CSimulatorNetMcast();
			simulator->Run(simName, argCount, argValue);
			delete simulator;
		}
		break;
	case '6':
		{
			CSimulatorFlow* simulator = new CSimulatorFlow();
			simulator->Run(simName, argCount, argValue);
			delete simulator;
		}
		break;
	default: printf("\n\nSimulator not found.");
	}
	
	printf("\n\n");

#ifdef _WINDOWS
	_CrtDumpMemoryLeaks();
#endif
}