#pragma once

#include "Model.h"
#include "Topo.h"
#include "Workload.h"
#include "Route.h"
#include "Router.h"
#include "Link.h"
#include "HostFlow.h"
#include "Info.h"

#define HOST_ID(index)				(index)
#define ROUTER_ID(index)			(index)

#define HOST_INDEX(id)				(id)
#define ROUTER_INDEX(id)			(id)

#define LINK_CORE_ID(index)			(0x10000000 | index)
#define LINK_ACCESS_ID(index)		(0x20000000 | index)

#define HOST_ADDR(index)			(index + this->routeNodes)
#define ROUTER_ADDR(index)			(index)

#define ROUTE_ADDRESS_INDEX(addr)	(addr)
#define ROUTE_ROUTER_INDEX(id)		(id)

class CModelFlow : public CModel, public CRoute, public CInfo
{
protected:
	// Simulation
	CSimHandler*		sim;
	CTopo*				topology;
	__uint32			topologyNumber;

	// Nodes
	__uint32			numHosts;
	__uint32			numGateways;
	CHostFlow**			hosts;
	CRouter**			routers;
	CLink**				linksCore;
	CLink**				linksAccess;
	__uint32*			gateways;
	__uint32*			routerAccessLinks;
	
	// Network
	__bitrate			bwAccessUpLink;
	__bitrate			bwAccessDownLink;
	__time				delayAccessUpLink;
	__time				delayAccessDownLink;
	__uint32			queueLink;
	
	// Routing
	__uint32			routeNodes;
	__uint32			routeDst;
	int**				route;

	// Data : bandwidth
	__uint64			dataBwTotal;
	double				dataBwAvg;

	// Data : bandwidth core
	__uint64			dataBwCoreTotal;
	double				dataBwCoreAvg;
	// Data : bandwidth access
	__uint64			dataBwAccessTotal;
	double				dataBwAccessAvg;
	// Data : bandwidth access uplink
	__uint64			dataBwAccessUpTotal;
	double				dataBwAccessUpAvg;
	// Data : bandwidth access downlink
	__uint64			dataBwAccessDownTotal;
	double				dataBwAccessDownAvg;

	// Data : bandwidth stream
	__uint64			dataBwStreamTotal;
	double				dataBwStreamAvg;
	// Data : bandwidth core stream
	__uint64			dataBwCoreStreamTotal;
	double				dataBwCoreStreamAvg;
	// Data : bandwidth access stream
	__uint64			dataBwAccessStreamTotal;
	double				dataBwAccessStreamAvg;
	// Data : bandwidth access uplink stream
	__uint64			dataBwAccessUpStreamTotal;
	double				dataBwAccessUpStreamAvg;
	// Data : bandwidth access downlink stream
	__uint64			dataBwAccessDownStreamTotal;
	double				dataBwAccessDownStreamAvg;

	// Data : bandwidth control
	__uint64			dataBwControlTotal;
	double				dataBwControlAvg;
	// Data : bandwidth core control
	__uint64			dataBwCoreControlTotal;
	double				dataBwCoreControlAvg;
	// Data : bandwidth access control
	__uint64			dataBwAccessControlTotal;
	double				dataBwAccessControlAvg;
	// Data : bandwidth access uplink control
	__uint64			dataBwAccessUpControlTotal;
	double				dataBwAccessUpControlAvg;
	// Data : bandwidth access downlink control
	__uint64			dataBwAccessDownControlTotal;
	double				dataBwAccessDownControlAvg;

public:
	CModelFlow(
		__time		maxTime,
		CTopo*		topology,
		__uint32	topologyNumber,
		__uint32	numHosts,
		__uint32	numGateways,
		__bitrate	bwAccessUpLink,
		__bitrate	bwAccessDownLink,
		__time		delayAccessUpLink,
		__time		delayAccessDownLink,
		__uint32	queueLink
		);
	virtual ~CModelFlow();

	virtual void					Init(CSimHandler* sim);
	virtual __uint32				Events();
	virtual CSimEvent*				Event(__uint32 index, __time& time);

	virtual void					Finalize();

	virtual inline __uint32			NumGroups() { return 0; }

	virtual inline CChannel*		Channel(__uint32 index) { return NULL; }
	virtual inline CChannel*		ChannelMulticast(__uint32 index) { return NULL; }
	virtual inline CChannel*		ChannelUnicast(__uint32 index) { return NULL; }

	virtual inline CWorkload*		Workload() { return NULL; }

	virtual int						Forward(CAddress router, CAddress dst);
};
