#pragma once

#include "Model.h"
#include "Topo.h"
#include "Workload.h"
#include "Route.h"
#include "InfoPushMulti.h"
#include "Router.h"
#include "Link.h"
#include "HostClientPushMulti.h"
#include "HostServerPushMulti.h"
#include "StreamSourceMpeg.h"
#include "Data.h"

#define HOST_ID(index)				(index)
#define ROUTER_ID(index)			(index)
#define SERVER_ID(index)			(index)

#define LINK_CORE_ID(index)			(0x10000000 | index)
#define LINK_ACCESS_ID(index)		(0x20000000 | index)
#define LINK_SERVER_ID(index)		(0x30000000 | index)

#define HOST_ADDR(index)			(index + this->routeNodes)
#define ROUTER_ADDR(index)			(index)
#define SERVER_ADDR(index)			(index + this->routeNodes + this->numHosts)

#define ROUTE_ADDRESS_INDEX(addr)	(addr)
#define ROUTE_ROUTER_INDEX(id)		(id)

class CModelPushMulti : public CModel, public CRoute, public CInfoPushMulti
{
protected:
	// Simulation
	char*					name;
	CSimHandler*			sim;
	CTopo*					topology;
	__uint32				topologyNumber;
	CWorkload*				workload;

	// Nodes
	__uint32				numHosts;
	__uint32				numServers;
	__uint32				numGateways;
	CHostClientPushMulti**	hosts;
	CHostServerPushMulti*	server;
	CRouter**				routers;
	CLink**					linksCore;
	CLink**					linksAccess;
	CLink*					linkServer;
	__uint32*				gateways;
	__uint32				gatewayServer;
	__uint32*				routerAccessLinks;
	
	// Network
	__uint32				bwAccessBins;
	__bitrate*				bwAccessUpLink;
	__bitrate*				bwAccessDownLink;
	double*					bwAccessRatio;
	__bitrate*				bwAccessHostUpLink;
	__bitrate*				bwAccessHostDownLink;
	__time					delayAccessUpLink;
	__time					delayAccessDownLink;
	__bitrate				bwServerUpLink;
	__bitrate				bwServerDownLink;
	__time					delayServerUpLink;
	__time					delayServerDownLink;
	__uint32				queueLink;
	__uint32				queueLinkServer;
	
	// Routing
	__uint32				routeNodes;
	__uint32				routeDst;
	int**					route;

	// Channels
	__uint32				numChannels;
	__uint32				numChannelsMcast;
	__uint32				numChannelsUcast;
	__uint32				numLayers;
	__uint32				numLayersMin;

	__bitrate				channelBw;
	__byte					channelFps;
	CChannel*				channelsMcast;
	CChannel*				channelsUcast;

	// Multicast
	CAddress*				mcastRp;

	// Stream
	CStreamSourceMpeg**		streamSources;
	__byte					mpegGopDistanceItoI;
	__byte					mpegGopDistanceItoP;

	__uint32				streamBufferMcastSize;
	__uint32				streamBufferMcastSizeHistory;
	__uint32				streamBufferMcastSizeBuffering;
	__uint32				streamBufferUcastSize;
	__uint32				streamBufferUcastSizeHistory;
	__uint32				streamBufferUcastSizeBuffering;
	__time					streamBufferUnderrunTimeout;
	double					streamBwMargin;

	// Bootstrap
	__uint32				bootQueryMax;
	__uint32				bootRefreshThreshold;
	__time					bootQueryTimeout;
	__time					bootRegisterDelay;

	// Data
	CData*					data;

	// Data : bandwidth
	__uint64				dataBwTotal;
	double					dataBwAvg;

	// Data : bandwidth core
	__uint64				dataBwCoreTotal;
	double					dataBwCoreAvg;
	// Data : bandwidth access
	__uint64				dataBwAccessTotal;
	double					dataBwAccessAvg;
	// Data : bandwidth access uplink
	__uint64				dataBwAccessUpTotal;
	double					dataBwAccessUpAvg;
	// Data : bandwidth access downlink
	__uint64				dataBwAccessDownTotal;
	double					dataBwAccessDownAvg;
	// Data : bandwidth server
	__uint64				dataBwServerTotal;
	double					dataBwServerAvg;

	// Data : bandwidth stream
	__uint64				dataBwStreamTotal;
	double					dataBwStreamAvg;
	// Data : bandwidth core stream
	__uint64				dataBwCoreStreamTotal;
	double					dataBwCoreStreamAvg;
	// Data : bandwidth access stream
	__uint64				dataBwAccessStreamTotal;
	double					dataBwAccessStreamAvg;
	// Data : bandwidth access uplink stream
	__uint64				dataBwAccessUpStreamTotal;
	double					dataBwAccessUpStreamAvg;
	// Data : bandwidth access downlink stream
	__uint64				dataBwAccessDownStreamTotal;
	double					dataBwAccessDownStreamAvg;
	// Data : bandwidth server stream
	__uint64				dataBwServerStreamTotal;
	double					dataBwServerStreamAvg;

	// Data : bandwidth control
	__uint64				dataBwControlTotal;
	double					dataBwControlAvg;
	// Data : bandwidth core control
	__uint64				dataBwCoreControlTotal;
	double					dataBwCoreControlAvg;
	// Data : bandwidth access control
	__uint64				dataBwAccessControlTotal;
	double					dataBwAccessControlAvg;
	// Data : bandwidth access uplink control
	__uint64				dataBwAccessUpControlTotal;
	double					dataBwAccessUpControlAvg;
	// Data : bandwidth access downlink control
	__uint64				dataBwAccessDownControlTotal;
	double					dataBwAccessDownControlAvg;
	// Data : bandwidth server control
	__uint64				dataBwServerControlTotal;
	double					dataBwServerControlAvg;

	// Data : multicast entries
	double					dataRouterMcastEntries;
	double					dataRouterMcastEntriesIgmp;
	double					dataRouterMcastEntriesPimSm;

	// Data : host statistics
	__uint32				dataCoderEncodedPackets;
	__uint32				dataCoderDecodedPackets;
	__uint32				dataCoderDiscardedPackets;

	__uint32				dataCoderEncodedFrames;
	__uint32				dataCoderDecodedFrames;
	__uint32				dataCoderDiscardedFrames;

	__uint32				dataClientRecvFrames;
	__uint32				dataClientDiscardedFrames;
	__uint32				dataClientExpectedFrames;

public:
	CModelPushMulti(
		char*		name,
		__time		maxTime,
		CTopo*		topology,
		__uint32	topologyNumber,
		CWorkload*	workload,
		__uint32	numHosts,
		__uint32	numGateways,
		__uint32	bwAccessBins,
		__bitrate*	bwAccessUpLink,
		__bitrate*	bwAccessDownLink,
		double*		bwAccessRatio,
		__time		delayAccessUpLink,
		__time		delayAccessDownLink,
		__bitrate	bwServerUpLink,
		__bitrate	bwServerDownLink,
		__time		delayServerUpLink,
		__time		delayServerDownLink,
		__uint32	queueLink,
		__uint32	queueLinkServer,
		__uint32	numChannelsMcast,
		__uint32	numChannelsUcast,
		__uint32	numLayers,
		__uint32	numLayersMin,
		__bitrate	channelBw,
		__byte		channelFps,
		__byte		mpegGopDistanceItoI,
		__byte		mpegGopDistanceItoP,
		__uint32	streamBufferMcastSize,
		__uint32	streamBufferMcastSizeHistory,
		__uint32	streamBufferMcastSizeBuffering,
		__uint32	streamBufferUcastSize,
		__uint32	streamBufferUcastSizeHistory,
		__uint32	streamBufferUcastSizeBuffering,
		__time		streamBufferUnderrunTimeout,
		double		streamBwMargin,
		__uint32	bootQueryMax,
		__uint32	bootRefreshThreshold,
		__time		bootQueryTimeout,
		__time		bootRegisterDelay
		);
	virtual ~CModelPushMulti();

	virtual void					Init(CSimHandler* sim);
	virtual __uint32				Events();
	virtual CSimEvent*				Event(__uint32 index, __time& time);
	
	virtual void					Finalize();

	virtual int						Forward(CAddress router, CAddress dst);

	virtual inline __uint32			NumGroups() { return this->numChannelsMcast; }

	virtual inline __uint32			NumChannels() { return this->numChannels; }
	virtual inline __uint32			NumChannelsMulticast() { return this->numChannelsMcast; }
	virtual inline __uint32			NumChannelsUnicast() { return this->numChannelsUcast; }
	virtual inline __uint32			NumLayers() { return this->numLayers; }
	virtual inline __uint32			NumLayersMin() { return this->numLayersMin; }
	virtual inline __byte			MpegGopSize() { return this->mpegGopDistanceItoI; }
	
	virtual inline CChannel*		Channel(__uint32 index);
	virtual inline CChannel*		ChannelMulticast(__uint32 index) { assert(index < this->numChannelsMcast); return &this->channelsMcast[index]; }
	virtual inline CChannel*		ChannelUnicast(__uint32 index) { assert(index < this->numChannelsUcast); return &this->channelsUcast[index]; }
	
	virtual inline __uint32			IndexGlobalToMulticast(__uint32 index) { assert(index < this->numChannelsMcast); return index; }
	virtual inline __uint32			IndexGlobalToUnicast(__uint32 index) { assert(index < this->numChannels); assert(index >= this->numChannelsMcast); return index - this->numChannelsMcast; }
	virtual inline __uint32			IndexMulticastToGlobal(__uint32 index) { assert(index < this->numChannelsMcast); return index; }
	virtual inline __uint32			IndexUnicastToGlobal(__uint32 index) { assert(index < this->numChannelsUcast); return index + this->numChannelsMcast; }

	virtual inline __uint32			StreamBufferMcastSize() { return this->streamBufferMcastSize; }
	virtual inline __uint32			StreamBufferMcastSizeHistory() { return this->streamBufferMcastSizeHistory; }
	virtual inline __uint32			StreamBufferMcastSizeBuffering() { return this->streamBufferMcastSizeBuffering; }
	virtual inline __uint32			StreamBufferUcastSize() { return this->streamBufferUcastSize; }
	virtual inline __uint32			StreamBufferUcastSizeHistory() { return this->streamBufferUcastSizeHistory; }
	virtual inline __uint32			StreamBufferUcastSizeBuffering() { return this->streamBufferUcastSizeBuffering; }
	virtual inline __time			StreamBufferUnderrunTimeout() { return this->streamBufferUnderrunTimeout; }
	virtual inline	double			StreamBwMargin() { return this->streamBwMargin; }

	virtual inline __uint32			BootQueryMax() { return this->bootQueryMax; }
	virtual inline __uint32			BootRefreshThreshold() { return this->bootRefreshThreshold; }
	virtual inline __time			BootQueryTimeout() { return this->bootQueryTimeout; }
	virtual inline __time			BootRegisterDelay() { return this->bootRegisterDelay; }

	virtual inline __uint16			PortStream() { return 0; }
	virtual inline __uint16			PortControl() { return 1; }

	virtual inline CWorkload*		Workload() { return this->workload; }

	virtual inline CStreamSource*	StreamSource(__uint32 index) { assert(index < this->numChannels); return this->streamSources[index]; }
};
