#pragma once

#include "Object.h"
#include "Codes.h"
#include "Address.h"
#include "Info.h"
#include "Link.h"
#include "Route.h"

#include "LayerIp.h"
#include "LayerIpLocal.h"
#include "LayerIpMcast.h"
#include "LayerIgmp.h"
#include "LayerUdp.h"

class CNode : public CObject
{
private:
	// Links
	__uint32									numLinks;
	__uint32									currentLink;
	
	CLink**										links;
	__uint32*									linkEntries;
	map<__uint32, __uint32>						linkIndices;

	// Stat
	__uint64									statPacketsRead;
	__uint64									statPacketsWrite;

	__uint64									statDataRead;
	__uint64									statDataWrite;

protected:
	// Global
	__uint32									id;
	CAddress									address;
	CInfo*										info;
	CRoute*										route;

	// Layers
	CLayerIp*									layerIp;
	CLayerIpLocal*								layerIpLocal;
	CLayerIpMcast*								layerIpMcast;
	CLayerUdp*									layerUdp;

	// Base layer delegate
	Delegate2<CNode, void, __uint32, CPacket*>*	delegateSend;

	// Base layer event
	Event2<void, __uint32, CPacket*>*			eventRecv;

public:
	CNode(
		__uint32		id,
		CSimHandler*	sim,
		CAddress		address,
		__uint32		numLinks,
		CRoute*			route,
		CInfo*			info
		);
	virtual ~CNode();

	// Global
	inline __uint32			Id() { return this->id; }
	inline CAddress			Address() { return this->address; }

	// Links
	void					AddLink(CLink* link);

	inline __uint32			NumLinks() { assert(this->numLinks == this->currentLink); return this->numLinks; }
	inline CLink*			Link(__uint32 index) { assert(index < this->currentLink); return this->links[index]; }
	inline __uint32			LinkEntry(__uint32 index) { assert(index < this->currentLink); return this->linkEntries[index]; }
	inline __uint32			LinkIndex(__uint32 id) { assert(this->linkIndices.find(id) != this->linkIndices.end()); return this->linkIndices.find(id)->second; }

	// Receive
	virtual ENetworkCode	Recv(CObject* sender, __uint32 entry, CPacket* packet);

	// Stat
	inline __uint64			StatPacketsRead() { return this->statPacketsRead; }
	inline __uint64			StatPacketsWrite() { return this->statPacketsWrite; }

	inline __uint64			StatDataRead() { return this->statDataRead; }
	inline __uint64			StatDataWrite() { return this->statDataWrite; }

	// Finalizer
	virtual void			Finalize();


private:
	void					Send(__uint32 entry, CPacket* packet);
};
