#include "Headers.h"
#include "Router.h"
#include "Console.h"

CRouter::CRouter(
	__uint32		id,
	CSimHandler*	sim,
	CAddress		address,
	__uint32		numLinks,
	CRoute*			route,
	CInfo*			info,
	CAddress*		rp
	) : CNode(id, sim, address, numLinks, route, info)
{
	// Initialize layers
	this->layerIgmp = new CLayerIgmp(this->sim, this->address, this->info->NumGroups(), CLayerIgmp::IGMP_ROUTER);
	assert_mem(this->layerIgmp);

	this->layerIpRoute = new CLayerIpRoute(
		this->sim,
		this->address,
		this->route
		);
	assert_mem(this->layerIpRoute);

	this->layerPimSm = new CLayerPimSm(
		this->sim,
		this->info->NumGroups(),
		this->address,
		rp,
		this->route
		);
	assert_mem(this->layerPimSm);

	// Create layer connections
		// IP layer <-> IGMP layer
	(*this->layerIp->EventRecv()) += this->layerIgmp->DelegateRecv();
	(*this->layerIgmp->EventSend()) += this->layerIp->DelegateSend();
		// IGMP layer <-> IP multicast layer
	(*this->layerIgmp->EventJoin()) += this->layerIpMcast->DelegateJoin();
	(*this->layerIgmp->EventLeave()) += this->layerIpMcast->DelegateLeave();
	(*this->layerIgmp->EventLocalJoin()) += this->layerIpMcast->DelegateLocalJoin();
	(*this->layerIgmp->EventLocalLeave()) += this->layerIpMcast->DelegateLocalLeave();
		// Layer IP <-> Layer IP route
	(*this->layerIp->EventRecv()) += this->layerIpRoute->DelegateRecv();
	(*this->layerIpRoute->EventSend()) += this->layerIp->DelegateSend();
		// Layer IGMP  <-> Layer PIM-SM
	(*this->layerIgmp->EventJoin()) += this->layerPimSm->DelegateJoin();
	(*this->layerIgmp->EventLeave()) +=  this->layerPimSm->DelegateLeave();
		// Layer IP <-> Layer PIM-SM
	(*this->layerIp->EventRecv()) += this->layerPimSm->DelegateRecv();
	(*this->layerPimSm->EventSend()) += this->layerIp->DelegateSend();
		// Layer PIM-SM <-> Layer IP multicast
	(*this->layerPimSm->EventJoin()) += this->layerIpMcast->DelegateJoin();
	(*this->layerPimSm->EventLeave()) += this->layerIpMcast->DelegateLeave();
}

CRouter::~CRouter()
{
	// Delete layers
	delete this->layerIgmp;
	delete this->layerIpRoute;
	delete this->layerPimSm;
}

void CRouter::Finalize()
{
	// Call base class finalizer
	CNode::Finalize();

	// Call the finalizer for each layer
	this->layerIgmp->Finalize();
	this->layerIpRoute->Finalize();
	this->layerPimSm->Finalize();
}