#include "Headers.h"
#include "SimulatorFlow.h"
#include "Sim.h"
#include "ModelFlow.h"
#include "TopoBrite.h"

#pragma warning(disable : 4996)

void CSimulatorFlow::Run(char* simName, int argc, char* argv[])
{
	printf("\n\nSimulator: FLOW (v. %s %s) (%s)\n", __DATE__, __TIME__, simName);

	if(argc < 3)
	{
		printf("\nSyntax is:\
			   \n\t1 - Input configuration file\
			   \n\t2 - Maximum time\
			   \n\t3 - Topology number(s)\
			   \n");
		return;
	}

	// Read input file
	char* fileInput = argv[1];

	FILE* fin;
	
	if(FILE_OPEN(fin, fileInput, "r")) { printf("\nCannot open configuration file: %s\n", fileInput); return; }
	
	__time		maxTime = atof(argv[2]);
	char		topoName[1024];
	char		topoFile[1024];
	__uint32	topoNums = argc - 3;
	__uint32	numHosts;
	__uint32	numGateways;
	__bitrate	bwAccessUpLink;
	__bitrate	bwAccessDownLink;
	__time		delayAccessUpLink;
	__time		delayAccessDownLink;
	__uint32	queueLink;

	fscanf(fin, "numHosts = %u\n", &numHosts);
	fscanf(fin, "numGateways = %u\n", &numGateways);
	fscanf(fin, "topoName = %s\n", topoName);
	fscanf(fin, "bwAccessUpLink = %lf\n", &bwAccessUpLink); 
	fscanf(fin, "bwAccessDownLink = %lf\n", &bwAccessDownLink); 
	fscanf(fin, "delayAccessUpLink = %lf\n", &delayAccessUpLink);
	fscanf(fin, "delayAccessDownLink = %lf\n", &delayAccessDownLink);
	fscanf(fin, "queueLink = %u\n", &queueLink);

	fclose(fin);

	for(__uint32 topoIndex = 0; topoIndex < topoNums; topoIndex++)
	{
		// Get the topology number
		__uint32 topoNum = atoi(argv[3 + topoIndex]);

		// Generate topology file
		sprintf(topoFile, "%s-%02u.brite", topoName, topoNum);

		printf("\nInput parameters are:\
			\n\t 1 - Maximum time              = %.0lf\
			\n\t 2 - Hosts                     = %u\
			\n\t 3 - Gateways                  = %u\
			\n\t 4 - Topology name             = %s\
			\n\t 5 - Topology file             = %s\
			\n\t 6 - Topology number           = %u\
			\n\t 7 - Bandwidth access uplink   = %.3lf\
			\n\t 7 - Bandwidth access downlink = %.3lf\
			\n\t 8 - Delay access uplink       = %.3lf\
			\n\t 9 - Delay access downlink     = %.3lf\
			\n\t10 - Queue link                = %u\
			\n\n",
			maxTime,
			numHosts,
			numGateways,
			topoName,
			topoFile,
			topoNum,
			bwAccessUpLink,
			bwAccessDownLink,
			delayAccessUpLink,
			delayAccessDownLink,
			queueLink
			);

		CTopo* topology = new CTopoBrite(
			topoFile
			);

		CModel* model = new CModelFlow(
			maxTime,
			topology,
			topoNum,
			numHosts,
			numGateways,
			bwAccessUpLink,
			bwAccessDownLink,
			delayAccessUpLink,
			delayAccessDownLink,
			queueLink
			);
		CSim* sim = new CSim(simName, model);

		time_t duration;

		sim->Run(duration);

		printf("\nSimulation finished in %ld seconds.", duration);

		delete sim;
		delete model;
		delete topology;
	}
}

