#include "Headers.h"
#include "SimulatorNetMcast.h"
#include "TopoBrite.h"
#include "MulticastTree.h"
#include "Shuffle.h"
#include "Rand.h"

#pragma warning(disable : 4996)

void CSimulatorNetMcast::Run(char* simName, int argc, char* argv[])
{
	printf("\n\nSimulator: NETWORK MULTICAST (v. %s %s) (%s)\n", __DATE__, __TIME__, simName);

	if(argc != 11)
	{
		printf("\nSyntax is:\
			   \n\t 1 - Topology name\
			   \n\t 2 - Output file\
			   \n\t 3 - Count edge nodes min\
			   \n\t 4 - Count edge nodes max\
			   \n\t 5 - Group size min\
			   \n\t 6 - Group size max\
			   \n\t 7 - Count trials edge nodes per topology\
			   \n\t 8 - Count trials groups per edge nodes set\
			   \n\t 9 - Min topology\
			   \n\t10 - Max topology\
			   \n");
		return;
	}

	// Input parameters
	char*		name = argv[1];
	char*		outName = argv[2];
	__uint32	numEdgeNodesMin = atoi(argv[3]);
	__uint32	numEdgeNodesMax = atoi(argv[4]);
	__uint32	groupMin = atoi(argv[5]);
	__uint32	groupMax = atoi(argv[6]);
	__uint32	numTrialsEdge = atoi(argv[7]);
	__uint32	numTrialsGroups = atoi(argv[8]);
	__uint32	minTopo = atoi(argv[9]);
	__uint32	maxTopo = atoi(argv[10]);

	printf("\nInput parameters are:\
		   \n\t 1 - Topology name                        : %s\
		   \n\t 2 - Output file                          : %s\
		   \n\t 3 - Count edge nodes min                   : %u\
		   \n\t 4 - Count edge nodes max                   : %u\
		   \n\t 5 - Group size min                       : %u\
		   \n\t 6 - Group size max                       : %u\
		   \n\t 7 - Count trials edge nodes per topology   : %u\
		   \n\t 8 - Count trials groups per edge nodes set : %u\
		   \n\t 9 - Min topology                         : %u\
		   \n\t10 - Max topology                         : %u\
		   \n\n",
		name,
		outName,
		numEdgeNodesMin,
		numEdgeNodesMax,
		groupMin,
		groupMax,
		numTrialsEdge,
		numTrialsGroups,
		minTopo,
		maxTopo);

	assert(groupMin < groupMax);

	char		topoFile[1024];
	__uint32	groups = groupMax - groupMin + 1;

	FILE* out = fopen(outName, "w");
	if(NULL == out) printf("\n\nCannot open file %s.", outName);

	// For each network topology
	for(__uint32 topo = minTopo; topo <= maxTopo; topo++)
	{
		// Generate topology file
		sprintf(topoFile, "%s-%02u.brite", name, topo);

		// Create topology
		CTopo* topology = new CTopoBrite(topoFile);

		// Create a multicast tree for the current topology
		CMulticastTree* tree = new CMulticastTree(topology, 0, 0);

		// For each number of edge nodes
		for(__uint32 nodes = numEdgeNodesMin; nodes <= numEdgeNodesMax; nodes++)
		{
			assert(nodes <= topology->Nodes());

			// For each random set of edge nodes
			for(__uint32 trialEdge = 0; trialEdge < numTrialsEdge; trialEdge++)
			{
				// Generate random edge nodes
				CShuffle shuffleEdge(topology->Nodes());

				// For each group trial
				for(__uint32 trialGroup = 0; trialGroup < numTrialsGroups; trialGroup++)
				{
					fprintf(out, "%u %u %u %u ", topo, nodes, trialEdge, trialGroup);

					printf("\nTopology : %s0-%02u Nodes : %u Edge set : %u Group set : %u", name, topo, nodes, trialEdge, trialGroup);

					// For each group size
					for(__uint32 group = groupMin; group <= groupMax; group++)
					{
						// Clear the tree
						tree->Clear();

						// For each host
						for(__uint32 index = 0; index < group; index++)
						{
							// Add host to a random edge node
							tree->Add(0, index, shuffleEdge[CRand::Generate(nodes-1)]);
						}

						fprintf(out, "%u ", tree->SizeCoreLast());
					}

					fprintf(out, "\n");
				}
			}
		}

		delete tree;

		delete topology;
	}

	fclose(out);
}