#include "Headers.h"
#include "SimulatorNetUcast.h"
#include "TopoBrite.h"
#include "Rand.h"

#pragma warning(disable : 4996)

void CSimulatorNetUcast::Run(char* simName, int argc, char* argv[])
{
	printf("\n\nSimulator: NETWORK UNICAST (v. %s %s) (%s)\n", __DATE__, __TIME__, simName);

	if(argc != 7)
	{
		printf("\nSyntax is:\
			   \n\t1 - Topology name\
			   \n\t2 - Output file\
			   \n\t3 - Count sources\
			   \n\t4 - Count destinations\
			   \n\t5 - Min topology\
			   \n\t6 - Max topology\
			   \n");
		return;
	}

	// Input parameters
	char*		name = argv[1];
	char*		outName = argv[2];
	__uint32	numSrc = atoi(argv[3]);
	__uint32	numDst = atoi(argv[4]);
	__uint32	minTopo = atoi(argv[5]);
	__uint32	maxTopo = atoi(argv[6]);

	printf("\nInput parameters are:\
		\n\t 1 - Topology name     = %s\
		\n\t 2 - Output file       = %s\
		\n\t 3 - Count sources       = %u\
		\n\t 4 - Count destinations  = %u\
		\n\t 5 - Min topology      = %u\
		\n\t 6 - Max topology      = %u\
		\n\n",
		name,
		outName,
		numSrc,
		numDst,
		minTopo,
		maxTopo);

	double		ucastLengthSum = 0;
	__uint32	ucastLengthNum = 0;
	char		topoFile[1024];

	FILE* out = fopen(outName, "w");
	if(NULL == out) printf("\n\nCannot open file %s.", outName);

	for(__uint32 topo = minTopo; topo <= maxTopo; topo++)
	{
		// Generate topology file
		sprintf(topoFile, "%s-%02u.brite", name, topo);

		CTopo* topology = new CTopoBrite(
			topoFile
			);

		for(__uint32 indexSrc = 0; indexSrc < numSrc; indexSrc++)
		{
			__uint32 src = CRand::Generate(topology->Nodes()-1);

			for(__uint32 indexDst = 0; indexDst < numDst; indexDst++)
			{
				__uint32 dst = CRand::Generate(topology->Nodes()-1);

				fprintf(out, "%u ", topology->Route()->Distance(src, dst));

				ucastLengthSum += topology->Route()->Distance(src, dst);
				ucastLengthNum ++;
			}
		}

		delete topology;
	}

	fclose(out);
	printf("\nAverage unicast path length: %.2lf", ucastLengthSum / ucastLengthNum);
}

