#include "Headers.h"
#include "SimulatorPull.h"
#include "Sim.h"
#include "ModelPull.h"
#include "TopoBrite.h"
#include "WorkloadFlat2Cat.h"

#pragma warning(disable : 4996)

void CSimulatorPull::Run(char* simName, int argc, char* argv[])
{
	printf("\n\nSimulator: PULL (v. %s %s)\n", __DATE__, __TIME__);

	if(argc < 8)
	{
		printf("\nSyntax is:\
			   \n\t1 - Input configuration file\
			   \n\t2 - Output name\
			   \n\t3 - Maximum time\
			   \n\t4 - Min multicast channels\
			   \n\t5 - Max multicast channels\
			   \n\t6 - Inc multicast channels\
			   \n\t7 - Topology number(s)\
			   \n");
		return;
	}

	// Read input file
	char* fileInput = argv[1];

	FILE* fin;
	
	if(FILE_OPEN(fin, fileInput, "r")) { printf("\nCannot open configuration file: %s\n", fileInput); return; }
	
	char*		name = argv[2];
	__time		maxTime = atof(argv[3]);
	char		topoName[1024];
	char		topoFile[1024];
	__uint32	topoNums = argc - 7;
	__uint32	numHosts;
	__uint32	numGateways;
	__uint32	bwAccessBins;
	__bitrate*	bwAccessUpLink;
	__bitrate*	bwAccessDownLink;
	double*		bwAccessRatio;
	__time		delayAccessUpLink;
	__time		delayAccessDownLink;
	__bitrate	bwServerUpLink;
	__bitrate	bwServerDownLink;
	__time		delayServerUpLink;
	__time		delayServerDownLink;
	__uint32	queueLink;
	__uint32	queueLinkServer;
	__uint32	numChannels;
	__uint32	numChannelsMcastMin = atoi(argv[4]);
	__uint32	numChannelsMcastMax = atoi(argv[5]);
	__uint32	numChannelsMcastInc = atoi(argv[6]);
	__bitrate	channelBw;
	__uint32	channelFps;
	__uint32	mpegGopDistanceItoI;
	__uint32	mpegGopDistanceItoP;
	__uint32	numPopular;
	double		probPopular;
	__uint32	streamBufferMcastSize;
	__uint32	streamBufferMcastSizeHistory;
	__uint32	streamBufferMcastSizeBuffering;
	__uint32	streamBufferUcastSize;
	__uint32	streamBufferUcastSizeHistory;
	__uint32	streamBufferUcastSizeBuffering;
	__time		streamBufferUnderrunTimeout;
	double		streamBwMargin;
	__uint32	bootQueryMax;
	__uint32	bootRefreshThreshold;
	__time		bootQueryTimeout;
	__time		bootRegisterDelay;
	__uint32	numPartners;
	__uint32	scheduleSize;
	__time		scheduleInterval;
	__uint32	streamSegmentSize;

	fscanf(fin, "numHosts = %u\n", &numHosts);
	fscanf(fin, "numGateways = %u\n", &numGateways);
	fscanf(fin, "topoName = %s\n", topoName);
	fscanf(fin, "numChannels = %u\n", &numChannels);
	fscanf(fin, "bwAccessBins = %u", &bwAccessBins);

	bwAccessUpLink = new __bitrate[bwAccessBins];
	bwAccessDownLink = new __bitrate[bwAccessBins];
	bwAccessRatio = new __bitrate[bwAccessBins];
	
	fscanf(fin, "\nbwAccessUpLink = "); 
	for(__uint32 index = 0; index < bwAccessBins; index++)
		fscanf(fin, "%lf", &bwAccessUpLink[index]);
	fscanf(fin, "\nbwAccessDownLink = "); 
	for(__uint32 index = 0; index < bwAccessBins; index++)
		fscanf(fin, "%lf", &bwAccessDownLink[index]);
	fscanf(fin, "\nbwAccessRatio = "); 
	for(__uint32 index = 0; index < bwAccessBins; index++)
		fscanf(fin, "%lf", &bwAccessRatio[index]);
	fscanf(fin, "\ndelayAccessUpLink = %lf\n", &delayAccessUpLink);
	fscanf(fin, "delayAccessDownLink = %lf\n", &delayAccessDownLink);
	fscanf(fin, "bwServerUpLink = %lf\n", &bwServerUpLink);
	fscanf(fin, "bwServerDownLink = %lf\n", &bwServerDownLink);
	fscanf(fin, "delayServerUpLink = %lf\n", &delayServerUpLink);
	fscanf(fin, "delayServerDownLink = %lf\n", &delayServerDownLink);
	fscanf(fin, "queueLink = %u\n", &queueLink);
	fscanf(fin, "queueLinkServer = %u\n", &queueLinkServer);
	fscanf(fin, "channelBw = %lf\n", &channelBw);
	fscanf(fin, "channelFps = %u\n", &channelFps);
	fscanf(fin, "mpegGopDistanceItoI = %u\n", &mpegGopDistanceItoI);
	fscanf(fin, "mpegGopDistanceItoP = %u\n", &mpegGopDistanceItoP);
	fscanf(fin, "numPopular = %u\n", &numPopular);
	fscanf(fin, "probPopular = %lf\n", &probPopular);
	fscanf(fin, "streamBufferMcastSize = %u\n", &streamBufferMcastSize);
	fscanf(fin, "streamBufferMcastSizeHistory = %u\n", &streamBufferMcastSizeHistory);
	fscanf(fin, "streamBufferMcastSizeBuffering = %u\n", &streamBufferMcastSizeBuffering);
	fscanf(fin, "streamBufferUcastSize = %u\n", &streamBufferUcastSize);
	fscanf(fin, "streamBufferUcastSizeHistory = %u\n", &streamBufferUcastSizeHistory);
	fscanf(fin, "streamBufferUcastSizeBuffering = %u\n", &streamBufferUcastSizeBuffering);
	fscanf(fin, "streamBufferUnderrunTimeout = %lf\n", &streamBufferUnderrunTimeout);
	fscanf(fin, "streamBwMargin = %lf\n", &streamBwMargin);
	fscanf(fin, "bootQueryMax = %u\n", &bootQueryMax);
	fscanf(fin, "bootRefreshThreshold = %u\n", &bootRefreshThreshold);
	fscanf(fin, "bootQueryTimeout = %lf\n", &bootQueryTimeout);
	fscanf(fin, "bootRegisterDelay = %lf\n", &bootRegisterDelay);
	fscanf(fin, "numPartners = %u\n", &numPartners);
	fscanf(fin, "scheduleSize = %u\n", &scheduleSize);
	fscanf(fin, "scheduleInterval = %lf\n", &scheduleInterval);
	fscanf(fin, "streamSegmentSize = %u\n", &streamSegmentSize);

	fclose(fin);

	for(__uint32 topoIndex = 0; topoIndex < topoNums; topoIndex++)
	{
		// Get the topology number
		__uint32 topoNum = atoi(argv[7 + topoIndex]);

		// Generate topology file
		sprintf(topoFile, "%s-%02u.brite", topoName, topoNum);

		printf("\nInput parameters are:\
			\n\t 1 - Maximum time                   = %.0lf\
			\n\t 2 - Hosts                          = %u\
			\n\t 3 - Gateways                       = %u\
			\n\t 4 - Topology name                  = %s\
			\n\t 5 - Topology file                  = %s\
			\n\t 6 - Topology number                = %u\
			\n\t 7 - Access bins                    = %u\
			",
			maxTime,
			numHosts,
			numGateways,
			topoName,
			topoFile,
			topoNum,
			bwAccessBins);

		printf("\n\t 8 - Bandwidth access uplink        = "); 
		for(__uint32 index = 0; index < bwAccessBins; index++)
			printf("%.0lf ", bwAccessUpLink[index]);
		printf("\n\t 9 - Bandwidth access downlink      = "); 
		for(__uint32 index = 0; index < bwAccessBins; index++)
			printf("%.0lf ", bwAccessDownLink[index]);
		printf("\n\t10 - Bandwidth access ratio         = "); 
		for(__uint32 index = 0; index < bwAccessBins; index++)
			printf("%.3lf ", bwAccessRatio[index]);

		printf("\
			\n\t11 - Delay access uplink            = %.3lf\
			\n\t12 - Delay access downlink          = %.3lf\
			\n\t13 - Bandwidth server uplink        = %.0lf\
			\n\t14 - Bandwidth server downlink      = %.0lf\
			\n\t15 - Delay server uplink            = %.3lf\
			\n\t16 - Delay server downlink          = %.3lf\
			\n\t17 - Queue link                     = %u\
			\n\t18 - Queue link server              = %u\
			\n\t19 - Count channels                 = %u\
			\n\t20 - Count channels multicast min   = %u\
			\n\t21 - Count channels multicast max   = %u\
			\n\t22 - Count channels multicast inc   = %u\
			\n\t23 - Channel bit rate               = %.0lf\
			\n\t24 - Channel FPS                    = %u\
			\n\t25 - MPEG I-to-T distance           = %u\
			\n\t26 - MPEG I-to-P distance           = %u\
			\n\t27 - Popular channels               = %u\
			\n\t28 - Popular probability            = %.2lf\
			\n\t29 - Buffer size (unicast)          = %u\
			\n\t30 - Buffer history (unicast)       = %u\
			\n\t31 - Buffering size (unicast)       = %u\
			\n\t32 - Buffer size (multicast)        = %u\
			\n\t33 - Buffer history (multicast)     = %u\
			\n\t34 - Buffering size (multicast)     = %u\
			\n\t35 - Buffer underrun timeout        = %lf\
			\n\t36 - Bandwidth margin               = %lf\
			\n\t37 - Boot query maximum size        = %u\
			\n\t38 - Boot refresh threshold         = %u\
			\n\t39 - Boot query timeout             = %lf\
			\n\t40 - Boot register delay            = %lf\
			\n\t41 - Number of partners             = %u\
			\n\t42 - Schedule size                  = %u\
			\n\t43 - Schedule interval              = %lf\
			\n\t44 - Stream segment size            = %u\
			\n\n",
			delayAccessUpLink,
			delayAccessDownLink,
			bwServerUpLink,
			bwServerDownLink,
			delayServerUpLink,
			delayServerDownLink,
			queueLink,
			queueLinkServer,
			numChannels,
			numChannelsMcastMin,
			numChannelsMcastMax,
			numChannelsMcastInc,
			channelBw,
			channelFps,
			mpegGopDistanceItoI,
			mpegGopDistanceItoP,
			numPopular,
			probPopular,
			streamBufferMcastSize,
			streamBufferMcastSizeHistory,
			streamBufferMcastSizeBuffering,
			streamBufferUcastSize,
			streamBufferUcastSizeHistory,
			streamBufferUcastSizeBuffering,
			streamBufferUnderrunTimeout,
			streamBwMargin,
			bootQueryMax,
			bootRefreshThreshold,
			bootQueryTimeout,
			bootRegisterDelay,
			numPartners,
			scheduleSize,
			scheduleInterval,
			streamSegmentSize
			);

		CTopo* topology = new CTopoBrite(
			topoFile
			);

		for(__uint32 numChannelsMcast = numChannelsMcastMin; numChannelsMcast <= numChannelsMcastMax; numChannelsMcast += numChannelsMcastInc)
		{
			if(numChannelsMcast > numChannels) continue;

			__uint32 numChannelsUcast = numChannels - numChannelsMcast;

			char currentSimName[100];
			sprintf(currentSimName, "[%s / %3u]", simName, numChannelsMcast);

			printf("\n\n%s : Multicast: %u Unicast: %u\n", currentSimName, numChannelsMcast, numChannelsUcast);

			CWorkload* workload = new CWorkloadFlat2Cat(
				numChannelsMcast + numChannelsUcast,
				numPopular,
				probPopular);

			CModel* model = new CModelPull(
				name,
				maxTime,
				topology,
				topoNum,
				workload,
				numHosts,
				numGateways,
				bwAccessBins,
				bwAccessUpLink,
				bwAccessDownLink,
				bwAccessRatio,
				delayAccessUpLink,
				delayAccessDownLink,
				bwServerUpLink,
				bwServerDownLink,
				delayServerUpLink,
				delayServerDownLink,
				queueLink,
				queueLinkServer,
				numChannelsMcast,
				numChannelsUcast,
				channelBw,
				channelFps,
				mpegGopDistanceItoI,
				mpegGopDistanceItoP,
				streamBufferMcastSize,
				streamBufferMcastSizeHistory,
				streamBufferMcastSizeBuffering,
				streamBufferUcastSize,
				streamBufferUcastSizeHistory,
				streamBufferUcastSizeBuffering,
				streamBufferUnderrunTimeout,
				bootQueryMax,
				bootRefreshThreshold,
				bootQueryTimeout,
				bootRegisterDelay,
				numPartners,
				scheduleSize,
				scheduleInterval,
				streamSegmentSize
				);
			CSim* sim = new CSim(currentSimName, model);

			time_t duration;

			sim->Run(duration);

			printf("\n%s : Simulation finished in %ld seconds.", currentSimName, duration);

			delete sim;
			delete model;
			delete workload;
		}

		delete topology;
	}
	
	delete[] bwAccessUpLink;
	delete[] bwAccessDownLink;
	delete[] bwAccessRatio;
}

