#include "Headers.h"
#include "StreamBootPushMulti.h"

CStreamBootPushMulti::CStreamBootPushMulti(__uint32 layers, CAddress root)
{
	assert(layers);

	this->layers = layers;
	this->root = root;

	this->hosts = new List[this->layers];
	assert_mem(this->hosts);
}

CStreamBootPushMulti::~CStreamBootPushMulti()
{
	delete[] this->hosts;
}

void CStreamBootPushMulti::Register(__uint32 layer, CAddress host, CAddress source)
{
	assert(layer < this->layers);
	this->hosts[layer].insert(pair<CAddress, CStreamBootEntry>(host, CStreamBootEntry(source)));
}

void CStreamBootPushMulti::Deregister(__uint32 layer, CAddress host)
{
	assert(layer < this->layers);
	this->hosts[layer].erase(host);
}

bool CStreamBootPushMulti::IsDependent(__uint32 layer, CAddress host, CAddress parent)
{
	assert(layer < this->layers);

	// For all ancestors from the parent to the root
	for(CAddress ancestor = parent; ancestor != this->root;)
	{
		// If the host is an ancestor of the parent, return true
		if(ancestor == host) return true;

		// Else, get the entry of the ancestor
		List::iterator entry = this->hosts[layer].find(ancestor);

		// If there is not entry for the ancestor (the ancestor is orphan), return true
		if(this->hosts[layer].end() == entry) return true;

		// Else, move the the source of the ancestor
		ancestor = entry->second.Source();
	}
	return false;
}