#pragma once

#include "StreamBufferLayer.h"

class CStreamBufferMulti
{
private:
	__uint32				numLayers;

	__uint32				size;
	__uint32				sizeHistory;
	__uint32				sizePlay;
	__byte					sizeGop;
	__uint32				sizeGopLayer;

	__uint32				currentIndex;

	CStreamBufferLayer**	buffers;

public:
	CStreamBufferMulti(
		__uint32	numLayers,
		__uint32	size,
		__uint32	sizeHistory,
		__byte	sizeGop
		);
	~CStreamBufferMulti();

	inline __uint32	IndexToLayer(__uint32 index) { return (index / this->sizeGop) % this->numLayers; }
	inline __uint32	IndexToLayerIndex(__uint32 index) { return this->sizeGop * (index / this->sizeGopLayer) + (index % this->sizeGop); }
	inline __uint32	LayerIndexToIndex(__uint32 layerIndex, __uint32 layer) { return this->sizeGopLayer * (layerIndex / this->sizeGop) + (layer * this->sizeGop) + (layerIndex % this->sizeGop); }
	inline __uint32	IndexToLayerBaseIndex(__uint32 index) { return this->sizeGop * (index / this->sizeGop); }

	void			Reset(__uint32 index);
	bool			Add(__uint32 layer, CStreamFrame frame);
	void			Shift();

	inline __uint32	Size() { return this->size; }
	inline __uint32	SizePlay() { return this->sizePlay; }
	inline __uint32	SizeHistory() { return this->sizeHistory; }

	inline __uint32	Count(__uint32 layer) { assert(layer < this->numLayers); return this->buffers[layer]->Count(); }
	inline __uint32	NumPlay(__uint32 layer) { assert(layer < this->numLayers); return this->buffers[layer]->NumPlay(); }
	inline __uint32	NumHistory(__uint32 layer) { assert(layer < this->numLayers); return this->buffers[layer]->NumHistory(); }

	bool			Has(__uint32 index);
	bool			HasCurrentIndex();

	inline __uint32	CurrentIndex() { return this->currentIndex; }
	inline __uint32	LayerIndex(__uint32 layer) { assert(layer < this->numLayers); return this->buffers[layer]->CurrentIndex(); }
	CStreamFrame*	CurrentFrame();

	bool			IsCurrentDecodable();
	bool			IsCurrentIndependent();
	bool			IsDecodable(__uint32 index);

	void			Log();
};

