#pragma once

#include "StreamClient.h"
#include "StreamBuffer.h"
#include "StreamBufferSegment.h"
#include "StreamDecoderFrame.h"
#include "InfoPull.h"
#include "Link.h"
#include "Timer.h"
#include "ConnectionLayer.h"
#include "ConnectionTag.h"

#include "StreamPullSender.h"
#include "StreamPullReceiver.h"

#include "StreamMessagePullBitmapRequest.h"
#include "StreamMessagePullBitmapResponse.h"
#include "StreamMessagePullSegmentRequest.h"
#include "StreamMessagePullSegmentResponse.h"

#include "StreamMessageBootPullRequest.h"
#include "StreamMessageBootPullResponse.h"
#include "StreamMessageBootPullRegister.h"
#include "StreamMessageBootPullDeregister.h"

class CStreamClientPull : public CStreamClient
{
private:
	typedef set<CAddress>						TNeighborList;
	typedef map<CAddress, CStreamPullSender*>	TRequestList;
	
	enum EClientState
	{
		STOPPED = 0,
		MCAST_STREAM_FIRST = 10,
		MCAST_STREAM_BUFFERING = 11,
		MCAST_STREAM_PLAY = 12,
		MCAST_STREAM_WAIT = 13,
		UCAST_STREAM_FIRST = 20,
		UCAST_STREAM_BUFFERING = 21,
		UCAST_STREAM_PLAY = 22
	};

	enum EScheduleState
	{
		SCHEDULE_IDLE = 0,
		SCHEDULE_BOOTSTRAP_REQUEST = 1,
		SCHEDULE_BOOTSTRAP_RESPONSE = 2,
		SCHEDULE_CONNECT_REQUEST = 3,
		SCHEDULE_CONNECT_RESPONSE = 4,
		SCHEDULE_BITMAP_REQUEST = 5,
		SCHEDULE_BITMAP_RESPONSE = 6,
		SCHEDULE_POST_BITMAP_RESPONSE = 7,
		SCHEDULE_SEGMENT_REQUEST = 8,
		SCHEDULE_SEGMENT_RESPONSE = 9,
		SCHEDULE_POST_SEGMENT_RESPONSE = 10
	};

	enum ERegistrationState
	{
		NOT_REGISTERED = 0,
		REGISTERED = 1
	};
	
	// State
	EClientState						stateClient;
	ERegistrationState					stateRegistration;
	EScheduleState						stateSchedule;

	// Simulator
	CInfoPull*							info;
	CAddress							address;

	// Channel
	CChannel*							channel;

	// Buffer
	CStreamBuffer*						bufferMcast;
	CStreamBufferSegment*				bufferUcast;

	// Session
	void								(CStreamClientPull::*processFrame)(CStreamFrame frame);
	__time								sessionTimeLastStart;
	__time								sessionTimeLastStop;
	__uint32							sessionFrameLastStart;
	__uint32							sessionFrameLastStop;

	// Multicast session
	__uint32							entry;			// Multicast interface (0 - as hosts have only one interface)

	// Unicast session
	CConnectionLayer*					connectionLayer;
	
	TNeighborList						neighbors;
	TRequestList						requestBitmaps;
	TRequestList						requestSegments;

	CStreamPullSender::TSenderList		sendersConnecting;
	CStreamPullSender::TSenderList		senders;
	CStreamPullReceiver::TReceiverList	receivers;

	__uint32							scheduleSegment;
	__uint8								scheduleNumCopiesLo[64];
	__uint8								scheduleNumCopiesHi[64];
	bool								scheduleBitmapLo[PULL_MAX_SENDERS][64];
	bool								scheduleBitmapHi[PULL_MAX_SENDERS][64];
	CStreamPullSender*					scheduleSenderLo[PULL_MAX_SENDERS][64];
	CStreamPullSender*					scheduleSenderHi[PULL_MAX_SENDERS][64];
	__uint8								scheduleSenderIndexLo[PULL_MAX_SENDERS][64];
	__uint8								scheduleSenderIndexHi[PULL_MAX_SENDERS][64];
	__time								scheduleDeadlineLo[PULL_MAX_SENDERS][64];
	__time								scheduleDeadlineHi[PULL_MAX_SENDERS][64];
	__bitrate							scheduleBw[PULL_MAX_SENDERS];

	CStreamPullSender*					scheduleSenders[64];
	__time								scheduleDeadlineDelta;


	// Timer
	CTimer<CStreamClientPull>*			timerPlayMcast;	// Playback timer (multicast channels)
	CTimer<CStreamClientPull>*			timerPlayUcast; // Playback timer (unicast channels)
	CTimer<CStreamClientPull>*			timerRegister;	// Registrap timer
	CTimer<CStreamClientPull>*			timerSchedule;	// Schedule timer

	// Delegates - receive 
	Delegate2<CStreamClientPull, void, CAddress, CStreamFrame>*
										delegateRecvFrameMcast;
	
	Delegate2<CStreamClientPull, void, CConnectionReceiver*, CPacket*>*
										delegateConnectionRecv;
	Delegate2<CStreamClientPull, bool, CAddress, CPacket*>*
										delegateConnectionAccept;
	Delegate1<CStreamClientPull, void, CConnectionSender*>*
										delegateConnectionAccepted;

	Delegate2<CStreamClientPull, void, CConnection*, CConnection::EOpenResult>*
										delegateConnectionReceiverOpen;
	Delegate2<CStreamClientPull, void, CConnection*, CConnection::ECloseResult>*
										delegateConnectionReceiverClose;

	Delegate2<CStreamClientPull, void, CConnection*, CConnection::EOpenResult>*
										delegateConnectionSenderOpen;
	Delegate2<CStreamClientPull, void, CConnection*, CConnection::ECloseResult>*
										delegateConnectionSenderClose;

	IDelegate2<void, CAddress, __uint32>*
										delegateIgmpJoin;
	IDelegate1<void, CAddress>*			delegateIgmpLeave;
	IDelegate5<void, __uint16, __uint16, CAddress, __byte, CPacket*>*
										delegateSend;
	IDelegate0<CLink*>*					delegateLink;

	// Decoder
	CStreamDecoderFrame*				decoderMcast;

	// Statistics
	__uint32							statRecvFrames;
	__uint32							statDiscardedFrames;
	__uint32							statPlayFrames;
	__uint32							statSuccessFrames[3];
	__uint32							statFailFrames[3];
	
	__uint32							statPlayFirstFrame;
	__uint32							statPlayLastFrame;

	__time								statTimeClientStart;
	__time								statTimeRecvStart;
	__time								statTimePlayStart;
	__time								statTimeFinish;
	__time								statTimeWait;

	__time								statSyncDelaySum;
	__uint32							statSyncDelayCount;

public:
	CStreamClientPull(
		CSimHandler*														sim,
		CInfoPull*															info,
		CAddress															address,
		IDelegate5<void, __uint16, __uint16, CAddress, __byte, CPacket*>*	delegateSend,
		IDelegate2<void, CAddress, __uint32>*								delegateIgmpJoin,
		IDelegate1<void, CAddress>*											delegateIgmpLeave,
		IDelegate0<CLink*>*													delegateLink,
		__uint32															entry
		);
	virtual ~CStreamClientPull();

	virtual void				Start(CChannel* channel);
	virtual void				Stop();

	virtual void				Recv(CAddress srcAddress, CAddress dstAddress, __uint16 srcPort, __uint16 dstPort, CPacket* packet);
	
	virtual inline __uint32		StatRecvFrames() { return this->statRecvFrames; }
	virtual inline __uint32		StatDiscardedFrames() { return this->statDiscardedFrames; }
	virtual inline __uint32		StatPlayFrames() { return this->statPlayFrames; }
	virtual inline __uint32*	StatSuccessFrames() { return this->statSuccessFrames; }
	virtual inline __uint32*	StatFailFrames() { return this->statFailFrames; }
	
	virtual inline __time		StatTimeClientStart() { return this->statTimeClientStart; }
	virtual inline __time		StatTimeRecvStart() { return this->statTimeRecvStart; }
	virtual inline __time		StatTimePlayStart() { return this->statTimePlayStart; }
	virtual inline __time		StatTimeFinish() { return this->statTimeFinish; }
	virtual inline __time		StatTimeWait() { return this->statTimeWait; }

	virtual inline __uint32		StatPlayFirstFrame() { return this->statPlayFirstFrame; }
	virtual inline __uint32		StatPlayLastFrame() { return this->statPlayLastFrame; }

	virtual inline __time		StatSyncDelay() { return this->statSyncDelayCount?(this->statSyncDelaySum / this->statSyncDelayCount):-1; }

	virtual void				Finalize();

private:
	void						StartMcast();
	void						StartUcast();

	void						StopMcast();
	void						StopUcast();

	void						RecvFrameMcast(CAddress address, CStreamFrame frame);

	void						ProcessFrameMcastFirst(CStreamFrame frame);
	void						ProcessFrameMcastBuffering(CStreamFrame frame);
	void						ProcessFrameMcastPlay(CStreamFrame frame);
	void						ProcessFrameMcastWait(CStreamFrame frame);

	void						ProcessFrameUcastFirst(CStreamFrame frame);
	void						ProcessFrameUcastBuffering(CStreamFrame frame);
	void						ProcessFrameUcastPlay(CStreamFrame frame);

	void						TimerPlayMcast(CTimerInfo* info);
	void						TimerPlayUcast(CTimerInfo* info);
	void						TimerRegister(CTimerInfo* info);
	void						TimerSchedule(CTimerInfo* info);

	void						ScheduleBootstrapRequest();
	void						ScheduleConnectRequest();
	void						ScheduleBitmapRequest();
	void						SchedulePostBitmapResponse();
	void						ScheduleSegmentRequest();

	void						RecvStream(CAddress dst, CPacketStream* packet);
	void						RecvMessage(CAddress src, CStreamMessage* message);
	void						RecvConnection(CAddress srcAddress, __uint16 srcPort, __uint16 dstPort, CPacketConnection* packet);

	void						RecvMessageBootResponse(CAddress src, CStreamMessageBootPullResponse* message);
	void						RecvMessageBitmapRequest(CAddress src, CStreamMessagePullBitmapRequest* message);
	void						RecvMessageBitmapResponse(CAddress src, CStreamMessagePullBitmapResponse* message);
	void						RecvMessageSegmentRequest(CAddress src, CStreamMessagePullSegmentRequest* message);
	void						RecvMessageSegmentResponse(CAddress src, CStreamMessagePullSegmentResponse* message);

	void						ConnectionRecv(CConnectionReceiver* receiver, CPacket* packet);
	bool						ConnectionAccept(CAddress src, CPacket* packet);
	void						ConnectionAccepted(CConnectionSender* sender);

	void						ConnectionReceiverOpen(CConnection* receiver, CConnection::EOpenResult result);
	void						ConnectionReceiverClose(CConnection* receiver, CConnection::ECloseResult result);

	void						ConnectionSenderOpen(CConnection* sender, CConnection::EOpenResult result);
	void						ConnectionSenderClose(CConnection* sender, CConnection::ECloseResult result);

	void						SendMessage(CAddress dst, CStreamMessage* message);

	void						UcastRegister();
	void						UcastDeregister();
};