#include "Headers.h"
#include "StreamMessageBootPushMultiResponse.h"

CStreamMessageBootPushMultiResponse::CStreamMessageBootPushMultiResponse(
	__uint32	stream,
	__uint32	layers
	) : CStreamMessage(CStreamMessage::STREAM_MESSAGE_BOOT_PUSH_MULTI_RESPONSE, 64, stream)
{
	assert(layers <= BOOT_MAX_LAYERS);

	this->type = STREAM;
	this->layer = 0;
	this->layers = layers;

	for(__uint32 index = 0; index < this->layers; index++)
	{
		this->count[index] = 0;
	}
}

CStreamMessageBootPushMultiResponse::CStreamMessageBootPushMultiResponse(
	__uint32	stream,
	__uint32	layer,
	__uint32	count
	) : CStreamMessage(CStreamMessage::STREAM_MESSAGE_BOOT_PUSH_MULTI_RESPONSE, 64 + count * 32, stream)
{
	this->type = LAYER;
	this->layer = layer;
	this->layers = 1;

	this->count[0] = count;
}

void CStreamMessageBootPushMultiResponse::Set(__uint32 layer, __uint32 count)
{
	assert(layer < this->layers);
	assert(count <= BOOT_MAX_HOSTS);

	this->size += (count * 32 - this->count[layer] * 32);
	this->count[layer] = count;
}

CPacket* CStreamMessageBootPushMultiResponse::Copy()
{
	CStreamMessageBootPushMultiResponse* packet;

	switch(this->type)
	{
	case STREAM: 
		packet = new CStreamMessageBootPushMultiResponse(
			this->stream,
			this->layers
			);
		for(__uint32 layer = 0; layer < this->layers; layer++)
		{
			packet->count[layer] = this->count[layer];
			for(__uint32 index = 0; index < this->count[layer]; index++)
				packet->hosts[layer][index] = this->hosts[layer][index];
		}
		break;
	case LAYER:
		packet = new CStreamMessageBootPushMultiResponse(
			this->stream,
			this->layer,
			this->count[0]
			);
		for(__uint32 index = 0; index < this->count[0]; index++)
			packet->hosts[0][index] = this->hosts[0][index];
		break;		
	}

	assert_mem(packet);
	return packet;
}