#include "Headers.h"
#include "StreamPullReceiver.h"
#include "PacketStreamFrame.h"

CStreamPullReceiver::CStreamPullReceiver(
	CSimHandler*		sim,
	CConnectionSender*	connection,
	__time				interval,
	__uint32			id,
	TReceiverList*		list
	)
{
	// Input
	this->connection = connection;
	this->interval = interval;

	// Timer
	this->timer = new CTimer<CStreamPullReceiver>(sim, this, &CStreamPullReceiver::Timer);
	assert_mem(this->timer);

	// List
	this->id = id;
	this->list = list;
	if(this->list)
	{
		assert(this->list->find(this->id) == this->list->end());
		this->list->insert(pair<__uint32, CStreamPullReceiver*>(this->id, this));
	}

	// Add this as tag to the connection
	this->connection->Tag(this);
}

CStreamPullReceiver::~CStreamPullReceiver()
{
	// Timer
	delete this->timer;

	// If a list is specified, remove the receiver from the list
	if(this->list)
	{
		assert(this->list->find(this->id) != this->list->end());
		this->list->erase(this->id);
	}

	// Remove this as tag from the connection
	assert(this->connection->Tag() == this);
	this->connection->Tag(NULL);
}

void CStreamPullReceiver::Send(CStreamFrame frame)
{
	// If the sending timer is not set
	if(!this->timer->IsSet())
	{
		// Send the frame through the connection
		
		// Create a stream packet
		CPacketStreamFrame* packet = new CPacketStreamFrame(frame);
		assert_mem(packet);

		this->connection->Send(packet);

		// Set the timer to block the transmission
		this->timer->SetAfter(this->interval);
	}
	else
	{
		// Add the packet to the buffer
		this->buffer.push(frame);
	}
}

void CStreamPullReceiver::Timer(CTimerInfo* info)
{
	// If the buffer is not empty
	if(!this->buffer.empty())
	{
		// Pop the next frame from the buffer
		CStreamFrame frame = this->buffer.front();
		this->buffer.pop();

		// Create a stream packet
		CPacketStreamFrame* packet = new CPacketStreamFrame(frame);
		assert_mem(packet);

		this->connection->Send(packet);

		// Set the timer to block the transmission
		this->timer->SetAfter(this->interval);
	}
}


void CStreamPullReceiver::List(TReceiverList* list)
{
	// If there is an old list, remove the sender from the list
	if(this->list)
	{
		assert(this->list->find(this->id) != this->list->end());
		this->list->erase(this->id);
	}

	// Change the list
	this->list = list;

	// If a list is specified, add the sender to the list
	if(this->list)
	{
		assert(this->list->find(this->id) == this->list->end());
		this->list->insert(pair<__uint32, CStreamPullReceiver*>(this->id, this));
	}
}