#include "Headers.h"
#include "StreamPullSender.h"

CStreamPullSender::CStreamPullSender(CAddress address, CConnectionReceiver* connection, TSenderList* list)
{
	this->address = address;
	this->connection = connection;
	this->list = list;
	this->type = CONNECTING_SENDER;

	// If a list is specified, add the sender to the list
	if(this->list)
	{
		assert(this->list->find(this->address) == this->list->end());
		this->list->insert(pair<CAddress, CStreamPullSender*>(this->address, this));
	}

	// Add this as tag to the connection
	this->connection->Tag(this);
}

CStreamPullSender::~CStreamPullSender()
{
	// If a list is specified, remove the sender from the list
	if(this->list)
	{
		assert(this->list->find(this->address) != this->list->end());
		this->list->erase(this->address);
	}

	// Remove this as tag from the connection
	assert(this->connection->Tag() == this);
	this->connection->Tag(NULL);
}

void CStreamPullSender::List(TSenderList* list)
{
	// If there is an old list, remove the sender from the list
	if(this->list)
	{
		assert(this->list->find(this->address) != this->list->end());
		this->list->erase(this->address);
	}

	// Change the list
	this->list = list;

	// If a list is specified, add the sender to the list
	if(this->list)
	{
		assert(this->list->find(this->address) == this->list->end());
		this->list->insert(pair<CAddress, CStreamPullSender*>(this->address, this));
	}
}