#pragma once

#include "ConnectionTag.h"
#include "ConnectionReceiver.h"
#include "StreamMessagePullBitmapResponse.h"

class CStreamPullSender : public CConnectionTag
{
public:
	typedef map<CAddress, CStreamPullSender*>	TSenderList;

	enum EType
	{
		CONNECTING_SENDER = 0,
		SENDER = 1
	};

	class CBitmapResponse
	{
	private:
		__uint32										first;
		__uint8											count;
		__uint64										bitmap;
		__bitrate										bw;
		CStreamMessagePullBitmapResponse::EPreference	pref;

	public:
		CBitmapResponse() { this->first = 0; this->count = 0; this->bitmap = 0; this->bw = 0; }
		CBitmapResponse(__uint32 first, __uint8 count, __uint64 bitmap, __bitrate bw, CStreamMessagePullBitmapResponse::EPreference pref)
		{ this->first = first; this->count = count; this->bitmap = bitmap; this->bw = bw; this->pref = pref; }
		~CBitmapResponse() { }

		inline __uint32											First() { return this->first; }
		inline __uint8											Count() { return this->count; }
		inline __uint64											Bitmap() { return this->bitmap; }
		inline __bitrate										Bw() { return this->bw; }
		inline CStreamMessagePullBitmapResponse::EPreference	Preference() { return this->pref; }

		inline void												Clear()
		{ this->first = 0; this->count = 0; this->bitmap = 0; this->bw = 0; }
	};

	class CSegmentResponse
	{
	private:
		__uint32	first;
		__uint8		count;
		__uint64	success;
		__uint64	fail;

	public:
		CSegmentResponse() { this->first = 0; this->count = 0; this->success = 0; this->fail = 0; }
		CSegmentResponse(__uint32 first, __uint8 count, __uint64 success, __uint64 fail) { this->first = first; this->count = count; this->success = success; this->fail = fail; }
		~CSegmentResponse() { }

		inline __uint32	First() { return this->first; }
		inline __uint8	Count() { return this->count; }
		inline __uint64	Success() { return this->success; }
		inline __uint64	Fail() { return this->fail; }

		inline void		Clear() { this->first = 0; this->count = 0; this->success = 0; this->fail = fail; }
	};

private:
	CAddress				address;
	CConnectionReceiver*	connection;
	EType					type;
	CBitmapResponse			bitmapResponse;
	CSegmentResponse		segmentResponse;
	TSenderList*			list;

public:
	CStreamPullSender(CAddress address, CConnectionReceiver* connection, TSenderList* list);
	virtual ~CStreamPullSender();

	inline CAddress				Address() { return this->address; }
	inline CConnectionReceiver*	Connection() { return this->connection; }
	inline EType				Type() { return this->type; }
	inline void					Type(EType type) { this->type = type; }
	inline CBitmapResponse		LastBitmapResponse() { return this->bitmapResponse; }
	inline void					SetBitmapResponse(CBitmapResponse bitmapResponse) { this->bitmapResponse = bitmapResponse; }
	inline CSegmentResponse		LastSegmentResponse() { return this->segmentResponse; }
	inline void					SetSegmentResponse(CSegmentResponse segmentResponse) { this->segmentResponse = segmentResponse; }
	
	void						List(TSenderList* list);
	inline TSenderList*			List() { return this->list; }
};
