#pragma once

#include "StreamServer.h"
#include "StreamBootPull.h"
#include "InfoPull.h"
#include "Timer.h"
#include "Link.h"
#include "PacketStream.h"
#include "ConnectionLayer.h"

#include "StreamPullSender.h"
#include "StreamPullReceiver.h"

#include "StreamMessageBootPullRequest.h"
#include "StreamMessageBootPullResponse.h"
#include "StreamMessageBootPullRegister.h"
#include "StreamMessageBootPullDeregister.h"

#include "StreamMessagePullBitmapRequest.h"
#include "StreamMessagePullBitmapResponse.h"
#include "StreamMessagePullSegmentRequest.h"
#include "StreamMessagePullSegmentResponse.h"

class CStreamServerPull : public CStreamServer
{
private:
	typedef map<__uint32, CStreamPullReceiver*>	TReceiverList;

	// Info
	CInfoPull*						info;

	// Bootstrap
	CStreamBootPull*				bootstraps;

	// Delegates
	IDelegate5<void, __uint16, __uint16, CAddress, __byte, CPacket*>*
									delegateSend;

	Delegate2<CStreamServerPull, void, CConnectionReceiver*, CPacket*>*
									delegateConnectionRecv;
	Delegate2<CStreamServerPull, bool, CAddress, CPacket*>*
									delegateConnectionAccept;
	Delegate1<CStreamServerPull, void, CConnectionSender*>*
									delegateConnectionAccepted;

	Delegate2<CStreamServerPull, void, CConnection*, CConnection::EOpenResult>*
									delegateConnectionSenderOpen;
	Delegate2<CStreamServerPull, void, CConnection*, CConnection::ECloseResult>*
									delegateConnectionSenderClose;

	IDelegate0<CLink*>*				delegateLink;

	// Connection
	CConnectionLayer*				connectionLayer;

	// Stream
	TReceiverList					receivers;

public:
	CStreamServerPull(
		CSimHandler*														sim,
		CInfoPull*															info,
		IDelegate5<void, __uint16, __uint16, CAddress, __byte, CPacket*>*	delegateSend,
		IDelegate0<CLink*>*													delegateLink
		);
	virtual ~CStreamServerPull();

	// Control
	void			Start();
	void			Stop();

	// Receive
	void			Recv(CAddress srcAddress, __uint16 srcPort, __uint16 dstPort, CPacket* packet);

	// Finalizer
	virtual void	Finalize();

private:
	void			Timer(CTimerInfo* info);

	
	void			RecvMessage(CAddress src, CStreamMessage* message);
	void			RecvMessageQuery(CAddress src, CStreamMessageBootPullRequest* message);
	void			RecvMessageRegister(CAddress src, CStreamMessageBootPullRegister* message);
	void			RecvMessageDeregister(CAddress src, CStreamMessageBootPullDeregister* message);
	void			RecvMessageBitmapRequest(CAddress src, CStreamMessagePullBitmapRequest* message);
	void			RecvMessageSegmentRequest(CAddress src, CStreamMessagePullSegmentRequest* message);

	void			RecvConnection(CAddress srcAddress, __uint16 srcPort, __uint16 dstPort, CPacketConnection* packet);

	void			SendMessage(CAddress dst, CStreamMessage* message);

	void			ConnectionRecv(CConnectionReceiver* receiver, CPacket* packet);
	bool			ConnectionAccept(CAddress src, CPacket* packet);
	void			ConnectionAccepted(CConnectionSender* sender);

	void			ConnectionSenderOpen(CConnection* sender, CConnection::EOpenResult result);
	void			ConnectionSenderClose(CConnection* sender, CConnection::ECloseResult result);
};
