#include "Headers.h"
#include "StreamServerPushMcast.h"

CStreamServerPushMcast::CStreamServerPushMcast(
	CSimHandler*									sim,
	CStreamSource*									source,
	IDelegate2<void, CAddress, CPacketStream*>*		delegateSendStream,
	IDelegate2<void, CAddress, __uint32>*			delegateIgmpJoin,
	IDelegate1<void, CAddress>*						delegateIgmpLeave,
	__uint32										entry
	) : CStreamServer(sim)
{
	// Set source
	this->source = source;

	// Set lower layer delegates
	this->delegateSendStream = delegateSendStream;
	this->delegateIgmpJoin = delegateIgmpJoin;
	this->delegateIgmpLeave = delegateIgmpLeave;

	// Create the server timer and hook it to the timer method
	this->timer = new CTimer<CStreamServerPushMcast>(
		sim,
		this,
		&CStreamServerPushMcast::Timer);
	assert_mem(this->timer);

	// Set the schedule rate (the channel fps)
	this->scheduleRate = this->source->Channel()->Fps();

	// Set the interface
	this->entry = entry;

	// Coders
	this->encoder = new CStreamEncoderFrame(
		this->delegateSendStream
		);
	assert_mem(this->encoder);
}

CStreamServerPushMcast::~CStreamServerPushMcast()
{
	// Timer
	delete this->timer;

	// Coders
	delete this->encoder;
}

void CStreamServerPushMcast::Start()
{
	// Check the server timer is stopped
	assert(!this->timer->IsSet());

	// Join the multicast group for this channel
	(*this->delegateIgmpJoin)(this->source->Channel()->Address(), this->entry);

	// Synchronize with the stream source
	this->source->Synchronize(this->sim->Time(), this->scheduleTime, this->scheduleFrame);

	// Set the delta frame to zero
	this->deltaFrame = 0;

	// If the schedule time is the current time, call the timer function
	if(this->scheduleTime == this->sim->Time())
		this->Timer(NULL);
	else	// Else, set the timer at the scheduled time
		this->timer->SetAt(this->scheduleTime);
}

void CStreamServerPushMcast::Stop()
{
	// If the timer is set, cancel
	if(this->timer->IsSet()) this->timer->Cancel();

	// Leave the multicast group for this channel
	(*this->delegateIgmpLeave)(this->source->Channel()->Address());
}

void CStreamServerPushMcast::Timer(CTimerInfo* info)
{
	// Process server timer

	// Fetch the frame from the source and send it to the encoder (the destination is the channel multicast address)
	this->encoder->Encode(this->source->Channel()->Address(), this->source->Fetch(this->scheduleFrame + this->deltaFrame));

	// Increment the delta frame
	this->deltaFrame++;

	// Set a new timer
	this->timer->SetAt(this->scheduleTime + ((__time)this->deltaFrame)/this->scheduleRate);
}

void CStreamServerPushMcast::Finalize()
{
	// Call base class finalizer
	CStreamServer::Finalize();
}