#pragma once

#include "StreamServer.h"

#include "Address.h"
#include "Timer.h"
#include "StreamBootPush.h"
#include "Shuffle.h"

#include "PacketStream.h"
#include "StreamEncoderFrame.h"

#include "StreamMessage.h"
#include "StreamMessagePushJoin.h"
#include "StreamMessagePushLeave.h"
#include "StreamMessageBootPushRequest.h"
#include "StreamMessageBootPushResponse.h"
#include "StreamMessageBootPushRegister.h"
#include "StreamMessageBootPushDeregister.h"

class CStreamServerPushUcast : public CStreamServer
{
private:
	// Stream
	set<CAddress>					receivers;
	CTimer<CStreamServerPushUcast>*	timer;
	CStreamSource*					source;

	// Bootstrap
	CStreamBootPush*				bootstrap;

	// Delegates
	IDelegate2<void, CAddress, CPacketStream*>*
									delegateSendStream;
	IDelegate2<void, CAddress, CStreamMessage*>*
									delegateSendMessage;

	// Global
	__byte							gopLength;

	__time							scheduleTime;
	__uint32						scheduleFrame;
	__uint32						scheduleRate;
	__uint32						deltaFrame;

	// Encoder
	CStreamEncoderFrame*			encoder;

public:
	CStreamServerPushUcast(
		CSimHandler*									sim,
		CStreamSource*									source,
		IDelegate2<void, CAddress, CPacketStream*>*		delegateSendStream,
		IDelegate2<void, CAddress, CStreamMessage*>*	delegateSendMessage
		);
	virtual ~CStreamServerPushUcast();

	// Control
	void			Start();
	void			Stop();

	// Receive
	void			Recv(CAddress src, CStreamMessage* message);

	// Finalizer
	virtual void	Finalize();

private:
	void			Timer(CTimerInfo* info);

	void			RecvMessageQuery(CAddress src, CStreamMessageBootPushRequest* message);
	void			RecvMessageRegister(CAddress src, CStreamMessageBootPushRegister* message);
	void			RecvMessageDeregister(CAddress src, CStreamMessageBootPushDeregister* message);

	void			RecvMessageJoin(CAddress src, CStreamMessagePushJoin* message);
	void			RecvMessageLeave(CAddress src, CStreamMessagePushLeave* message);
};
