#pragma once

#include "StreamServer.h"

#include "Address.h"
#include "Timer.h"
#include "StreamBootPushMulti.h"
#include "Shuffle.h"

#include "PacketStream.h"
#include "StreamEncoderFrame.h"

#include "StreamMessage.h"
#include "StreamMessagePushMultiJoin.h"
#include "StreamMessagePushMultiLeave.h"
#include "StreamMessageBootPushMultiRequest.h"
#include "StreamMessageBootPushMultiResponse.h"
#include "StreamMessageBootPushMultiRegister.h"
#include "StreamMessageBootPushMultiDeregister.h"

class CStreamServerPushUcastMulti : public CStreamServer
{
private:
	// Stream
	CStreamSource*							source;
	set<CAddress>*							receivers;
	CTimer<CStreamServerPushUcastMulti>*	timer;

	// Bootstrap
	CStreamBootPushMulti*					bootstrap;

	// Delegates
	IDelegate2<void, CAddress, CPacketStream*>*
											delegateSendStream;
	IDelegate2<void, CAddress, CStreamMessage*>*
											delegateSendMessage;

	// Global
	__uint32								layers;

	__byte									gopLength;

	__time									scheduleTime;
	__uint32								scheduleFrame;
	__uint32								scheduleRate;
	__uint32								deltaFrame;

	// Encoder
	CStreamEncoderFrame*					encoder;

public:
	CStreamServerPushUcastMulti(
		CSimHandler*									sim,
		CStreamSource*									source,
		IDelegate2<void, CAddress, CPacketStream*>*		delegateSendStream,
		IDelegate2<void, CAddress, CStreamMessage*>*	delegateSendMessage,
		__uint32										layers,
		__byte											gopLength
		);
	virtual ~CStreamServerPushUcastMulti();

	// Control
	void			Start();
	void			Stop();

	// Receive
	void			Recv(CAddress src, CStreamMessage* message);

	// Finalizer
	virtual void	Finalize();

private:
	void			Timer(CTimerInfo* info);

	void			RecvMessageQuery(CAddress src, CStreamMessageBootPushMultiRequest* message);
	void			RecvMessageQueryStream(CAddress src, CStreamMessageBootPushMultiRequest* message);
	void			RecvMessageQueryLayer(CAddress src, CStreamMessageBootPushMultiRequest* message);

	void			RecvMessageRegister(CAddress src, CStreamMessageBootPushMultiRegister* message);
	void			RecvMessageDeregister(CAddress src, CStreamMessageBootPushMultiDeregister* message);

	void			RecvMessageJoin(CAddress src, CStreamMessagePushMultiJoin* message);
	void			RecvMessageLeave(CAddress src, CStreamMessagePushMultiLeave* message);
};
