#include "Headers.h"
#include "StreamSourceMpeg.h"

CStreamSourceMpegFrameTemplate::CStreamSourceMpegFrameTemplate(
	EFrameType	type,
	__uint32	size,
	char		anchorP,
	char		anchorB
	)
{
	this->type = type;
	this->size = size;
	this->anchorP = anchorP;
	this->anchorB = anchorB;
}

CStreamSourceMpeg::CStreamSourceMpeg(
	CChannel*	channel,
	__byte		gopDistanceItoI,
	__byte		gopDistanceItoP,
	__byte		segmentSize
	) : CStreamSource(channel)
{
	assert(gopDistanceItoI > 0);
	assert(gopDistanceItoP > 0);
	assert(gopDistanceItoI >= gopDistanceItoP);

	this->gopDistanceItoI = gopDistanceItoI;
	this->gopDistanceItoP = gopDistanceItoP;
	this->segmentSize = segmentSize;

	// Generate frame template
	this->gopTemplate = new CStreamSourceMpegFrameTemplate[this->gopDistanceItoI];
	assert_mem(this->gopTemplate);

	// I frame
	this->gopTemplate[0] = CStreamSourceMpegFrameTemplate(
		FRAME_I,
		(__uint32)ceil(this->channel->Bw()/this->channel->Fps()),
		0,
		0);

	// P frames
	for(char indexP = this->gopDistanceItoP; indexP < this->gopDistanceItoI; indexP += this->gopDistanceItoP)
	{
		this->gopTemplate[indexP] = CStreamSourceMpegFrameTemplate(
			FRAME_P,
			(__uint32)ceil(this->channel->Bw()/this->channel->Fps()),
			-((char)this->gopDistanceItoP),	// Previous anchor
			0);
	}

	// B frames
	for(char indexP = 0; indexP < this->gopDistanceItoI; indexP += this->gopDistanceItoP)
	{
		for(char indexB = indexP + 1; indexB < indexP + this->gopDistanceItoP; indexB++)
		{
			this->gopTemplate[indexB] = CStreamSourceMpegFrameTemplate(
				FRAME_B,
				(__uint32)ceil(this->channel->Bw()/this->channel->Fps()),
				-((char)(indexB - indexP)), // Previous anchor
				(indexP + this->gopDistanceItoP - indexB) // Next anchor
				);
		}
	}
}

CStreamSourceMpeg::~CStreamSourceMpeg()
{
	delete[] this->gopTemplate;
}

CStreamFrame CStreamSourceMpeg::Fetch(__uint32 index)
{
	// Generate frame

	// Calculate GOP index
	__uint32 idx = index % this->gopDistanceItoI;

	return CStreamFrame(
		this->channel->Id(),
		this->gopTemplate[idx].type,
		index,
		this->gopTemplate[idx].anchorP,
		this->gopTemplate[idx].anchorB,
		this->gopTemplate[idx].size,
		index / this->segmentSize,
		index % this->segmentSize,
		this->segmentSize
		);
}
