#include "Headers.h"
#include "TopoBrite.h"

CTopoBrite::CTopoBrite(const char* fileName)
{
	this->numNodes = 0;
	this->numEdges = 0;

	this->nodes = NULL;
	this->edges = NULL;

	this->Read(fileName);

	this->route = new CTopoBriteRoute(this->numNodes, this->numEdges, this->nodes, this->edges);
	assert(this->route);
}

CTopoBrite::~CTopoBrite()
{	
	if(this->nodes)
	{
		for(__uint32 index = 0; index < this->numNodes; index++) delete this->nodes[index];
		delete[] this->nodes;
	}
	if(this->edges)
	{
		for(__uint32 index = 0; index < this->numEdges; index++) delete this->edges[index];
		delete[] this->edges;
	}
	delete this->route;
}

void CTopoBrite::Read(const char* fileName)
{
	// Open file
	FILE* file = NULL;
	if(FILE_OPEN(file, fileName, "r"))
	{
		printf("\nCannot open Brite topology file: %s", fileName);
		exit(-1);
	}

	// Read header
#ifdef _WINDOWS
	if(fscanf_s(file, "Topology: ( %u Nodes, %u Edges )\n", &this->numNodes, &this->numEdges) != 2)
		throw "I/O Error";
	if(fscanf_s(file, "Model (%u - %s  ",
		&this->model,
		this->modelName,
		32) != 2)
		throw "I/O Error";
#else
	if(fscanf(file, "Topology: ( %u Nodes, %u Edges )\n", &this->numNodes, &this->numEdges) != 2)
		throw "I/O Error";
	if(fscanf(file, "Model (%u - %s  ",
		&this->model,
		this->modelName) != 2)
		throw "I/O Error";
#endif

	__uint32 numNodes;

    double bwMin;
    double bwMax;

	switch(model)
	{
	case TOPO_BRITE_MODEL_WAXMAN:
#ifdef _WINDOWS
		if(fscanf_s(file, "%u %u %u %u %u %lf %lf %u %u %lf %lf \n\n",
			&numNodes,
			&this->hs,
			&this->ls,
			&this->nodePlacement,
			&this->nodeLinks,
			&this->alpha,
			&this->beta,
			&this->growthType,
			&this->bwDist,
			&bwMin,
			&bwMax) != 11) throw "I/O error";
#else
		if(fscanf(file, "%u %u %u %u %u %lf %lf %u %u %lf %lf \n\n",
			&numNodes,
			&this->hs,
			&this->ls,
			&this->nodePlacement,
			&this->nodeLinks,
			&this->alpha,
			&this->beta,
			&this->growthType,
			&this->bwDist,
			&bwMin,
			&bwMax) != 11) throw "I/O error";
#endif
		break;
	default: throw "Not supported";
	}
	assert(this->numNodes == numNodes);

	this->bwMax = (__bitrate)ceil(bwMax);
	this->bwMin = (__bitrate)ceil(bwMin);

	// Read nodes
#ifdef _WINDOWS
	if(fscanf_s(file, "Nodes: ( %u )\n", &numNodes) != 1) throw "I/O error";
#else
	if(fscanf(file, "Nodes: ( %u )\n", &numNodes) != 1) throw "I/O error";
#endif
	assert(this->numNodes == numNodes);

	this->nodes = new CTopoBriteNode*[this->numNodes];
	assert(this->nodes);

	__uint32 id;
	__uint32 xpos;
	__uint32 ypos;
	__uint32 inDegree;
	__uint32 outDegree;
	int as;
	char type[16];

	for(__uint32 index = 0; index < this->numNodes; index++)
	{
#ifdef _WINDOWS
		if(fscanf_s(file, "%u %u %u %u %u %d %s\n", &id, &xpos, &ypos, &inDegree, &outDegree, &as, type, 16) != 7) throw "I/O error";
#else
		if(fscanf(file, "%u %u %u %u %u %d %s\n", &id, &xpos, &ypos, &inDegree, &outDegree, &as, type) != 7) throw "I/O error";
#endif
		assert(id == index);
		assert(inDegree == outDegree);

		this->nodes[index] = new CTopoBriteNode(
			index,
			xpos,
			ypos,
			inDegree
			);
		assert(this->nodes[index]);
	}

	// Read edges
	__uint32 numEdges;
#ifdef _WINDOWS
	if(fscanf_s(file, "\n\nEdges: ( %u )\n", &numEdges) != 1) throw "I/O error";
#else
	if(fscanf(file, "\n\nEdges: ( %u )\n", &numEdges) != 1) throw "I/O error";
#endif
	assert(this->numEdges == numEdges);

	this->edges = new CTopoBriteEdge*[this->numEdges];
	assert(this->edges);

	__uint32 from;
	__uint32 to;
	double length;
	double delay;
	double bw;
	int asFrom;
	int asTo;
	char val;

	for(__uint32 index = 0; index < this->numEdges; index++)
	{
#ifdef _WINDOWS
		if(fscanf_s(file, "%u %u %u %lf %lf %lf %d %d %s %c\n", &id, &from, &to, &length, &delay, &bw, &asFrom, &asTo, type, 16, &val) != 10) throw "I/O error";
#else
		if(fscanf(file, "%u %u %u %lf %lf %lf %d %d %s %c\n", &id, &from, &to, &length, &delay, &bw, &asFrom, &asTo, type, &val) != 10) throw "I/O error";
#endif
		assert(id == index);

		if(bw > this->bwMax) bw = (double)this->bwMax;
		if(bw < this->bwMin) bw = (double)this->bwMin;

		// Create the edge
		this->edges[index] = new CTopoBriteEdge(index, from, to, length, delay / 1000, (__bitrate)ceil(bw));
		assert(this->edges[index]);

		// Set the edge for the nodes
		this->nodes[from]->SetEdge(index);
		this->nodes[to]->SetEdge(index);
	}

	// Close file
	fclose(file);
}

//void CTopoBrite::AddUnicast(__uint32 src, __uint32 dst, __bitrate bw, __uint32& length)
//{
//	// Add a flow from the specified source to destination
//	length = 0;
//	for(__uint32 nextNode = src; nextNode != dst;)
//	{
//		CTopoBriteEdge* edge = this->edges[this->route->NextEdge(nextNode, dst)];
//		assert(edge);
//
//		edge->AddFlow(bw);
//
//		assert(this->bwUsed + bw <= this->bwTotal);
//		this->bwUsed += bw;
//
//		nextNode = this->route->NextNode(nextNode, dst);
//
//		length++;
//	}
//}
//
//void CTopoBrite::RemoveUnicast(__uint32 src, __uint32 dst, __bitrate bw)
//{
//	// Remove a flow from the specified source to destination
//	for(__uint32 nextNode = src; nextNode != dst;)
//	{
//		CTopoBriteEdge* edge = this->edges[this->route->NextEdge(nextNode, dst)];
//		assert(edge);
//
//		edge->RemoveFlow(bw);
//
//		assert(this->bwUsed >= bw);
//		this->bwUsed -= bw;
//
//		nextNode = this->route->NextNode(nextNode, dst);
//	}
//}
//
//void CTopoBrite::AddMulticast(__uint32 group, __uint32 src, __uint32 dst, __bitrate bw, int& mcastSizeDelta, int& mcastGroupDelta)
//{
//	// Add a multicast flow from the specified source to destination
//	mcastSizeDelta = 0;
//
//	// Calculate the group size delta: if the destination node already in the group then the delta is 0, otherwise 1
//	CTopoBriteNode* node = this->nodes[dst];
//	assert(node);
//
//	mcastGroupDelta = (node->McastAccess(group)?0:1);
//
//	// Start from the destination towards the source
//	__uint32 nextNode = dst;
//	for(; nextNode != src;)
//	{
//		node = this->nodes[nextNode];
//		assert(node);
//
//		// Add the group to the node
//		assert(this->mcastUsed >= node->Mcast());
//		this->mcastUsed -= node->Mcast();
//		__uint32 con = node->AddMulticast(group, nextNode == dst);
//		this->mcastUsed += node->Mcast();
//
//		// There must be at least one group entry at the node
//		assert(con);
//
//		// If the added group is the first for this node, also add the flow to the edge going from node
//		if(1 == con)
//		{
//			CTopoBriteEdge* edge = this->edges[this->route->NextEdge(nextNode, src)];
//			assert(edge);
//
//			assert(this->bwUsed + bw <= this->bwTotal);
//			this->bwUsed += bw;
//
//			edge->AddFlow(bw);
//
//			mcastSizeDelta++;
//
//			// Select next node towards the source
//			nextNode = this->route->NextNode(nextNode, src);
//		}
//		else
//			// The node has more group entries: stop here
//			break;
//	}
//
//	// If the next node is the source
//	if(src == nextNode)
//	{
//		// Add the group to the node
//		assert(this->mcastUsed >= this->nodes[src]->Mcast());
//		this->mcastUsed -= this->nodes[src]->Mcast();
//		this->nodes[src]->AddMulticast(group, nextNode == dst);
//		this->mcastUsed += this->nodes[src]->Mcast();
//	}
//}
//
//void CTopoBrite::RemoveMulticast(__uint32 group, __uint32 src, __uint32 dst, __bitrate bw, int& mcastSizeDelta, int& mcastGroupDelta)
//{
//	// Remove a multicast flow from the specified source to destination
//	mcastSizeDelta = 0;
//	
//	// Calculate the group size delta: if the destination node has the mcast greater than 1 then the delta is 0, otherwise -1
//	CTopoBriteNode* node = this->nodes[dst];
//	assert(node);
//	
//	mcastGroupDelta = (node->McastAccess(group) > 1)?0:1;
//
//	// Start from the destination towards the source
//	__uint32 nextNode = dst;
//	for(;nextNode != src;)
//	{
//		node = this->nodes[nextNode];
//		assert(node);
//
//		// Remove the group entry from the node
//		assert(this->mcastUsed >= node->Mcast());
//		this->mcastUsed -= node->Mcast();
//		__uint32 con = node->RemoveMulticast(group, nextNode == dst);
//		this->mcastUsed += node->Mcast();
//
//		// If the node does not have the group (no more destinations)
//		// also remove the group from the link
//		if(0 == con)
//		{
//			CTopoBriteEdge* edge = this->edges[this->route->NextEdge(nextNode, src)];
//			assert(edge);
//
//			assert(this->bwUsed >= bw);
//			this->bwUsed -= bw;
//
//			edge->RemoveFlow(bw);
//
//			mcastSizeDelta++;
//
//			// Select next node towards the source
//			nextNode = this->route->NextNode(nextNode, src);
//		}
//		else
//			break;
//	}
//
//	// If the next node is the source
//	if(src == nextNode)
//	{
//		// Remove the group from the node
//		assert(this->mcastUsed >= this->nodes[src]->Mcast());
//		this->mcastUsed -= this->nodes[src]->Mcast();
//		this->nodes[src]->RemoveMulticast(group, nextNode == dst);
//		this->mcastUsed += this->nodes[src]->Mcast();
//	}
//}
//
//void CTopoBrite::Clear()
//{
//	this->bwUsed = 0;
//	this->mcastUsed = 0;
//
//	for(__uint32 index = 0; index < this->numNodes; index++)
//		this->nodes[index]->Clear();
//	for(__uint32 index = 0; index < this->numEdges; index++)
//		this->edges[index]->Clear();
//}