#pragma once

#include "Topo.h"
#include "TopoBriteNode.h"
#include "TopoBriteEdge.h"
#include "TopoBriteRoute.h"

#define TOPO_BRITE_MODEL_WAXMAN	1

class CTopoBrite : public CTopo
{
private:
	__uint32				model;
	char					modelName[32];
	
	__uint32			hs;
	__uint32			ls;

	// Model parameters (Waxman)
	double					alpha;
	double					beta;
	__uint32			nodePlacement;
	__uint32			growthType;
	__uint32			bwDist;
	__uint32			nodeLinks;
	__bitrate				bwMin;
	__bitrate				bwMax;

	__uint32			numNodes;
	__uint32			numEdges;
	
	// Nodes
	CTopoBriteNode**		nodes;

	// Edges
	CTopoBriteEdge**		edges;

	// Route
	CTopoBriteRoute*		route;

public:
	CTopoBrite(const char* fileName);
	~CTopoBrite();

	inline __uint32		Nodes() { return this->numNodes; }
	inline CTopoNode*		Node(__uint32 index) { assert(this->nodes); return this->nodes[index]; }

	inline __uint32		Edges() { return this->numEdges; }
	inline CTopoEdge*		Edge(__uint32 index) { assert(this->edges); return this->edges[index]; }

	inline CTopoRoute*		Route() { return this->route; }

	inline __byte			Model() { return this->model; }
	inline char*			ModelName() { return this->modelName; }
	
	inline __uint32		Hs() { return this->hs; }
	inline __uint32		Ls() { return this->ls; }
	inline double			Alpha() { return this->alpha; }
	inline double			Beta() { return this->beta; }
	inline __uint32		NodePlacement() { return this->nodePlacement; }
	inline __uint32		GrowthType() { return this->growthType; }
	inline __uint32		BwDist() { return this->bwDist; }
	inline __uint32		NodeLinks() { return this->nodeLinks; }

protected:
	void					Read(const char* fileName);
	void					GenerateRoutes();
};
